{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

Initial Autor   : Marcus Fuchs

Last Modified at: 05. February 2003
Last Modified by: Marcus Fuchs

You may retrieve the latest version of this file at the MFLib home page,
located at http://www.mfhome.net/mflib/
-------------------------------------------------------------------------------}

unit MFAbout;

{******************************************************************************}
{                                                                              }
{  This Form is intended to be used from an other unit (MFDialogs)             }
{                                                                              }
{******************************************************************************}

// remove the period to enable Debug settings (see MFLib.inc)
{.$DEFINE DEBUG}

{$I MFLib.inc}

interface { ------------------------------------------------------ interface - }

uses
  Windows, SysUtils, Classes, Controls, Forms, Dialogs, StdCtrls,      { VCL   }
  ExtCtrls,                                                            { VCL   }
  MFLabels, MFTypes;                                                   { MFLib }

type
  TMFAboutDlg = class(TForm)
    AppIconImg: TImage;
    ProductL: TLabel;
    VersionL: TLabel;
    LegalCopyrightL: TLabel;
    HomepageL: TLabel;
    EmailL: TLabel;
    AppCopyrightL: TLabel;
    AppHomepageL: TMFUrlLabel;
    AppEmailL: TMFUrlLabel;
    OKBtn: TButton;
    Bevel1: TBevel;
    AppProductnameL: TMFLabel;
    AppVersionL: TMFLabel;
    AppCommentL: TMFLabel;
    procedure FormCreate(Sender: TObject);
  private
    FStyle: TMFAboutDlgStyle;
    procedure DoClassic(var ctop: Integer);
    procedure DoModern(var ctop: Integer);
    procedure Paint; override;
    procedure SetTitle(Value: String);
    procedure SetProduct(Value: String);
    procedure SetVersion(Value: String);
    procedure SetCopyright(Value: String);
    procedure SetHomepage(Value: String);
    procedure SetEmail(Value: String);
    procedure SetInfo(Value: String);
  protected
  public
    property Title: String write SetTitle;
    property Product: String write SetProduct;
    property Version: String write SetVersion;
    property Copyright: String write SetCopyright;
    property Homepage: String write SetHomepage;
    property Email: String write SetEmail;
    property Info: String write SetInfo;
    property Style: TMFAboutDlgStyle write FStyle;
  end;

var
  MFAboutDlg: TMFAboutDlg;


implementation { -------------------------------------------- implementation - }

{$R *.DFM}

uses
  MFStrRes;

const
  space_beforclassicprog = 8;
  space_afterclassicprog = 8;
  left_classicapp = 144;
  left_classicver = 144;
  space_beformodernprog = 16;
  space_aftermodernprog = 1;
  left_modernapp = 56;
//  left_modernver = 56 + 8;

  space_afterversion = 8;
  space_afterimage = 8;
  space_aftercopyright = 8;
  space_afterhome = 3;
  space_beforline = 8;
  space_afterline = 8;
  space_btn = 8;

procedure TMFAboutDlg.FormCreate(Sender: TObject);
begin
  ProductL.Caption := srProductnameL;
  VersionL.Caption := srVersionL;
  LegalCopyrightL.Caption := srLegalCopyrightL;
  HomepageL.Caption := srHomepageL;
  EmailL.Caption := srEmailL;
  OKBtn.Caption := srAboutOKBtn;

  Title := srMFAboutDlg;
  AppIconImg.Picture.Icon := Application.Icon;
end;

procedure TMFAboutDlg.DoClassic(var ctop: Integer);
begin
  AppProductnameL.ParentFont := True;
  AppProductnameL.TextStyle := tsNone;
  ProductL.Visible := True;
  AppVersionL.TextStyle := tsNone;

  ctop := space_beforclassicprog;
  ProductL.Top := ctop;
  AppProductnameL.Top := ctop;
  ctop := ctop + (ctop - AppProductnameL.Top) + AppProductnameL.Height +
    space_afterclassicprog;

  AppProductnameL.Left := left_classicapp;

  VersionL.Visible := AppVersionL.Visible;
  if AppVersionL.Visible then begin
    VersionL.Top := ctop;
    AppVersionL.Top := ctop;
    ctop := AppVersionL.Top + AppVersionL.Height + space_afterversion;

    AppVersionL.Left := left_classicver;
  end
  else
    ctop := AppIconImg.Top + AppIconImg.Height + space_afterimage;
end;

procedure TMFAboutDlg.DoModern(var ctop: Integer);
begin
  AppProductnameL.TextStyle := tsHeavyRaised;
  AppProductnameL.Font.Size := 12;
  ProductL.Visible := False;
  AppVersionL.TextStyle := tsHeavyLowered;

  ctop := space_beformodernprog;
  AppProductnameL.Top := ctop;
  ctop := ctop + (ctop - AppProductnameL.Top) + AppProductnameL.Height +
    space_aftermodernprog;

  AppProductnameL.Left := left_modernapp;

  VersionL.Visible := False;
  if AppVersionL.Visible then begin
    AppVersionL.Top := ctop;
    ctop := AppVersionL.Top + AppVersionL.Height + space_afterversion;

    AppVersionL.Left := AppProductnameL.Left + AppProductnameL.Width -
      AppVersionL.Width;
  end
  else
    ctop := AppIconImg.Top + AppIconImg.Height + space_afterimage;
end;

procedure TMFAboutDlg.Paint;
var
  ctop: Integer;
begin
  AppVersionL.Visible := AppVersionL.Caption <> '';
  AppCopyrightL.Visible := AppCopyrightL.Caption <>'';
  LegalCopyrightL.Visible := AppCopyrightL.Visible;
  AppHomepageL.Visible := AppHomepageL.Caption <> '';
  HomepageL.Visible := AppHomepageL.Visible;
  AppEmailL.Visible := AppEmailL.Caption <> '';
  EmailL.Visible := AppEmailL.Visible;
  AppCommentL.Visible := AppCommentL.Caption <> '';
  Bevel1.Visible := AppCommentL.Visible;

  ctop := 0;
  if FStyle = dsClassic then
    DoClassic(ctop)
  else
    DoModern(ctop);

  if AppCopyrightL.Visible then begin
    LegalCopyrightL.Top := ctop;
    AppCopyrightL.Top := ctop;
    ctop := ctop + (ctop - LegalCopyrightL.Top) + LegalCopyrightL.Height +
      space_aftercopyright;
  end;
  if AppHomepageL.Visible then begin
    HomepageL.Top := ctop;
    AppHomepageL.Top := ctop;
    ctop := ctop + (ctop - HomepageL.Top) + HomepageL.Height + space_afterhome;
  end;
  if AppEmailL.Visible then begin
    EmailL.Top := ctop;
    AppEmailL.Top := ctop;
    ctop := ctop + (ctop - EmailL.Top) + EmailL.Height;
  end;
  if Bevel1.Visible then begin
    ctop := ctop + space_beforline;
    Bevel1.Top := ctop;
    ctop := ctop + (ctop - Bevel1.Top) + Bevel1.Height + space_afterline;
  end;
  if AppCommentL.Visible then begin
    AppCommentL.Top := ctop;
    if (AppCommentL.Left + AppCommentL.Width + 8 >= OKBtn.Left) then
      ctop := ctop + (ctop - AppCommentL.Top) + AppCommentL.Height + space_btn
  end
  else
    ctop := ctop + space_btn;
  OKBtn.Top := ctop;

  Self.ClientHeight := ctop + (ctop - OKBtn.Top) + OKBtn.Height + space_btn;

  inherited Paint;
end;

procedure TMFAboutDlg.SetTitle(Value: String);
begin
  if (Value <> Self.Caption) and (Value <> '')then
    Self.Caption := Value
end;

procedure TMFAboutDlg.SetProduct(Value: String);
begin
  if Value <> AppProductnameL.Caption then
    AppProductnameL.Caption := Value;
end;

procedure TMFAboutDlg.SetVersion(Value: String);
begin
  if Value <> AppVersionL.Caption then
    AppVersionL.Caption := Value;
end;

procedure TMFAboutDlg.SetCopyright(Value: String);
begin
  if Value <> AppCopyrightL.Caption then
    AppCopyrightL.Caption := Value;
end;

procedure TMFAboutDlg.SetHomepage(Value: String);
begin
  if Value <> AppHomepageL.Caption then begin
    AppHomepageL.Caption := Value;
    AppHomepageL.Url := AppHomepageL.Caption;
  end;
end;

procedure TMFAboutDlg.SetEmail(Value: String);
begin
  if Value <> AppEmailL.Caption then begin
    AppEmailL.Caption := Value;
    AppEmailL.Url := 'mailto:' + AppEmailL.Caption;
  end;
end;

procedure TMFAboutDlg.SetInfo(Value: String);
begin
  if Value <> AppCommentL.Caption then begin
    AppCommentL.Caption := Value;
    AppCommentL.Width := self.ClientWidth - AppCommentL.Left;
  end;
end;

end.
