{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

Initial Autor   : Marcus Fuchs

Last Modified at: 01. February 2003
Last Modified by: Marcus Fuchs

You may retrieve the latest version of this file at the MFLib home page,
located at http://www.mfhome.net/mflib/
-------------------------------------------------------------------------------}

unit MFButtons;

interface { ------------------------------------------------------ interface - }

// remove the period to enable Debug settings (see MFLib.inc)
{.$DEFINE DEBUG}

{$I MFLib.inc}

uses
  Windows, Messages, Classes, Controls, Graphics, Forms, Buttons, StdCtrls,
  ExtCtrls, Menus, Dialogs, SysUtils,                                  { VCL   }
  MFTypes;                                                             { MFLib }

{Added some presets to make things easier}
type
  TMFPreset = (None, GoldCoast, LimeBox, Aluminum, SunRise, BrassedOff,
               Encarta, {Square, }Select, AOL);

type
  TMFButton = class(TCustomControl)
  private
    FActive: Boolean;                // True if Control has Focus
    FIsMouseOver: Boolean;           // True if Mouse is over Control

    FCancel: Boolean;
    FDefault: Boolean;

    FMenuPoppedUp: Boolean;
    FDropdownMenu: TPopupMenu;

    FMarkerOffset: Integer;

    FBorderWidth: Integer;           // Size of 3D Frame
    FFrameLayout: TMFFrameLayout;

    FColors: TMFMouseColors;         // Colors of ButtonFace
    FImage: TBitmap;
    FBorderColors: TMFBorderColors;  // Colors of ButtonBorders
    FTextColors: TMFTextColors;      // Colors of Text

    FShowHoverColors: Boolean;       // True if colors should change if hovered
    FShowHoverFrame: Boolean;        // True if darkshadow should be drawn to an flatstyled button if hovered
    FShowFocus: Boolean;             // True if indicator rectangle should be drawn if we have focus

    FTextStyle: TMFTextStyle;
    FAlignment: TAlignment;

    FOnMouseEnter: TNotifyEvent;     // Make the Mouse Enter and Leave available
    FOnMouseLeave: TNotifyEvent;

    mUp: Boolean;        // Flags for Mouse Up or Down
    Clicking: Boolean;   // To catch a real button click...
    bm: TBitmap;         // WorkHorse internal bmp
    Caption2: string;    // ABB caption w/out ampersand!

    procedure WMSetFocus(Var Msg: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(Var Msg: TWMKillFocus); message WM_KILLFOCUS;
    // I had to add this to get the Caption to change while designing
    procedure CMMouseEnter(var Msg: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Msg: TMessage); message CM_MOUSELEAVE;
    // Processing these messages allows events to fire when mouse enters or leaves
    procedure CMTextChanged(var Msg: TMessage); message CM_TEXTCHANGED;
    procedure CMEnabledChanged(var Msg: TMessage); message CM_ENABLEDCHANGED;
    procedure CMDialogKey(var Msg: TCMDialogKey); message CM_DIALOGKEY;
    procedure CMFocusChanged(var Msg: TCMFocusChanged); message CM_FOCUSCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure WMEraseBkgnd(Var Msg: TMessage); message WM_ERASEBKGND;
    // I added this to help reduce flicker and speed things up.
    procedure KeyAccel(var Msg: TCMDialogChar); message CM_DIALOGCHAR;

    procedure SetDefault(Value: Boolean);

    procedure SetDropdownMenu(Value: TPopupMenu);

    procedure SetBorderWidth(Value: Integer);          // Set size of 3D frame
    procedure SetFrameLayout(Value: TMFFrameLayout);

    procedure SetImage(Value: TBitmap);
    procedure SetShowFocus(Value: Boolean);            // Draw Rect when we have focus T/F
    procedure SetShowHoverColors(Value: Boolean);
    procedure SetShowHoverFrame(Value: Boolean);
    procedure SetStyle(Value: TMFTextStyle);           // Set Caption 3D style
    procedure SetAlignment(Value: TAlignment);         // Select caption justification

    procedure DoRaise(offset: Integer);                // This draws the 3D Caption 'raised'
    procedure DoInset(offset: Integer);                // This draws te 3D caption 'inset'
    procedure DoNorm(offset: Integer);                 // This draws the caption with no 3D

    procedure DoNone(fr, fg, fb: Integer);
    procedure DoImage;

    procedure DrawPopupMark;
    procedure OnChanged(Sender: TObject);
    procedure OnTextColorChanged(Sender: TObject);
    procedure ChangedImage(Sender: TObject);
  protected
    { I added the following to allow the Enter key to behave as a Click. }
    procedure KeyPress(var Key: Char); override;
    procedure Notification(AComponent: TComponent;
       Operation: TOperation); override;
    procedure SetEnabled(value: Boolean);
    procedure SetBorderColors(Value: TMFBorderColors);
    procedure SetTextColors(Value: TMFTextColors);
    procedure SetColors(Value: TMFMouseColors);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    
    procedure Paint; override;  // Override Paint - entire control is painted.
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
       X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
       X, Y: Integer); override;
    // Mouse up and down are what I use to swap colors
    procedure Click; override;
  published
    { This first batch of stuff is all the new properties I added for this
      component.  As long as the properties are part of the usual types, you
      don't have to define your own property editor. I love Delphi. }
    property Cancel: Boolean read FCancel write FCancel default False;
    property Default: Boolean read FDefault write SetDefault default False;

    property BorderWidth: Integer read FBorderWidth write SetBorderWidth
       default 1;
    property FrameLayout: TMFFrameLayout read FFrameLayout write SetFrameLayout
       default flFlat;

    property BorderColors: TMFBorderColors read FBorderColors
       write SetBorderColors;
    property TextColors: TMFTextColors read FTextColors write SetTextColors;
    property Colors: TMFMouseColors read FColors write SetColors;
    property Image: TBitmap read FImage write SetImage;

    property ShowFocus: Boolean read FShowFocus write SetShowFocus default True;
    property ShowHoverColors: Boolean read FShowHoverColors
       write SetShowHoverColors default True;
    property ShowHoverFrame: Boolean read FShowHoverFrame
       write SetShowHoverFrame default True;

    property TextStyle: TMFTextStyle read FTextStyle write SetStyle
       default tsRaised;
    property Alignment: TAlignment read FAlignment write SetAlignment
       default taCenter;

    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
    property DropdownMenu: TPopupMenu read FDropdownMenu write SetDropdownMenu;

    property Align;
    property Caption;
    property Visible;
    property Enabled;
    property Font;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseMove;
    property OnMouseDown;
    property OnMouseUp;
    property OnClick;
  end;

{This is what it is all for}
type
  TMFGradButton = class(TMFButton)
  private
    FGradient: TMFGradient;
    FSwapColor: Boolean;           // True if start-end colors should be swaped on button press
    FPreset: TMFPreset;            // Variable for presets

    TmpClr: TColor;                // Holding area for Color during Mouse Down.

    function Muldv(a, b, c: integer): longint;

    procedure SetSwap(Value: Boolean);          // Swap the start-end true-false
    procedure SetPreset(Value: TMFPreset);      // Set preset values

    procedure DoHorizontal(fr, fg, fb, dr, dg, db: Integer); // These all draw
    procedure DoVertical(fr, fg, fb, dr, dg, db: Integer);   // the gradients.
    procedure DoElliptic(fr, fg, fb, dr, dg, db: Integer);   // The fr fg fb
    procedure DoRectangle(fr, fg, fb, dr, dg, db: Integer);  // dr dg db are
    procedure DoVertCenter(fr, fg, fb, dr, dg, db: Integer); // color values.
    procedure DoHorizCenter(fr, fg, fb, dr, dg, db: Integer);//
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetGradient(Value: TMFGradient);
    procedure Paint; override;  // Override Paint - entire control is painted.
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
       X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
       X, Y: Integer); override;
    // Mouse up and down are what I use to swap gradient colors
    procedure Click; override;
  published
    property Gradient: TMFGradient read FGradient write SetGradient;
    property SwapColors: Boolean read FSwapColor write SetSwap default True;
    property Presets: TMFPreset read FPreset write SetPreset default None;
  end;


implementation { -------------------------------------------- implementation - }

{$R ..\Resources\MFButtons.dcr}

{ - TMFButton ---------------------------------------------------------------- }

constructor TMFButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMenuPoppedUp := False;
  FMarkerOffset := 0;                        // offset if drop down menu used
  FCancel := False;
  FDefault := False;

  FBorderWidth := 1;                         // Default border size
  FFrameLayout := flFlat;

  FBorderColors := TMFBorderColors.Create;
  BorderColors.OnChanged := OnChanged;

  FTextColors := TMFTextColors.Create;
  TextColors.OnChanged := OnTextColorChanged;
  Font.Color := FTextColors.Colors.Color;    

  FColors := TMFMouseColors.Create;
  FColors.Color := $00C0DCC0;
  FColors.HoverColor := clBtnFace;
  FColors.DownColor := clBtnFace;
  Colors.OnChanged := OnChanged;

  bm := TBitmap.Create;           // Create the internal BMP
  bm.Canvas.Pen.Style := psSolid; // These may already be the default,
  bm.Canvas.Pen.Mode := pmCopy;   // but just in case...

  Height := 25;                   // Default button size
  Width := 75;

  FIsMouseOver := False;
  FShowHoverColors := True;
  FShowHoverFrame := True;

  FImage := TBitmap.Create;
  FImage.OnChange := ChangedImage;

  mUp := True;                    // Mouse is UP to start with
  ControlStyle := ControlStyle + [csOpaque];   // This reduces flicker
  FShowFocus := True;             // Draw focusrect when we have focus
  FTextStyle := tsRaised;         // Raised '3d' text
  TabStop := True;                // Made this the default so button can have Focus

  FAlignment := taCenter;         // Default is center justify
end;

destructor TMFButton.Destroy;
begin
  FBorderColors.free;
  FTextColors.free;
  FColors.free;
  bm.Free;
  FImage.Free;
  inherited Destroy;
end;

{Everything pretty much happens in the Paint procedure; a lot of it is farmed
 out to other procedures just so the logic in the paint procedure can be
 understood a little better.  Here's the basic process for creating a gradient:
 Get the separate R, G, and B values of the BeginClr Color.  then, find the
 difference between that and the R, G, and B values of the EndClr Color.  Using
 these two sets of values, draw a series of 'shapes' using a series of colors
 between the begin and end.  After the gradient is drawn, add a frame and the
 Caption.}
procedure TMFButton.Paint;
var
  FromR, FromG, FromB: Integer; // These are the separate color values for RGB
  CursorPos: TPoint;
  rct, crct: TRect;             // Rectangle used to draw frame around button
  offset: Integer;              // Used for Caption location during Mouse Down
  i: Integer;                   // loop variable to iterate Caption for &. ABB
{ To speed things up and reduce flicker, I use a Bitmap to draw the button in
  its entirety, then BitBlt it to the canvas of the control. }
begin
  //inherited;                  // PROBABLY DON'T NEED THIS
  bm.Canvas.Font := Font;       // Set the Bitmap font to match control font
  bm.Width := Width;            // Set BMP dimensions to match control's
  bm.Height := Height;
  rct := Rect(0, 0, Width, Height);      // Set rectangle size for later use

  if Assigned(DropDownMenu) then DrawPopupMark;

  // By setting the Brush style to Clear, it will draw without overlaying bkgrnd
  bm.Canvas.Brush.Style := bsClear;  // Gradient is done, time for Hilite-Shadow

  if mUp = True then begin  // if Mouse up do normal (or hover) button
    offset := 0;            // offset is used to move the Caption down a notch when the button is pressed.

    crct.TopLeft := Parent.ClientToScreen(Point(Left, Top));
    crct.BottomRight := Parent.ClientToScreen(Point(Left + Width, Top + Height));
    GetCursorPos(CursorPos);
    FIsMouseOver := PtInRect(crct, CursorPos);

    crct := GetClientRect;
    if FIsMouseOver and FShowHoverColors then begin
      FromR := ColorToRGB(FColors.HoverColor) and $000000ff;   // Strip out separate RGB values
      FromG := (ColorToRGB(FColors.HoverColor) shr 8) and $000000ff;
      FromB := (ColorToRGB(FColors.HoverColor) shr 16) and $000000ff;
      DoNone(FromR, FromG, FromB);

      if (FShowHoverFrame and (FrameLayout = flFlat)) or
         (FrameLayout = flNormal) then begin
        Frame3D(bm.Canvas, rct, FBorderColors.LightColors.HoverColor,
          FBorderColors.DkShadowColors.HoverColor, FBorderWidth);  // Draw mousehover frame
        bm.Canvas.Pen.Color := FBorderColors.ShadowColors.HoverColor;
        bm.Canvas.MoveTo(crct.Left+FBorderWidth, crct.Bottom-1-FBorderWidth);
        bm.Canvas.LineTo(crct.Right-1-FBorderWidth, crct.Bottom-1-FBorderWidth);
        bm.Canvas.LineTo(crct.Right-1-FBorderWidth, crct.Top-1+FBorderWidth);
      end
      else
        Frame3D(bm.Canvas, rct, FBorderColors.LightColors.HoverColor,
          FBorderColors.ShadowColors.HoverColor, FBorderWidth);  // Draw mousehover frame
    end
    else begin
      FromR := ColorToRGB(FColors.Color) and $000000ff;   // Strip out separate RGB values
      FromG := (ColorToRGB(FColors.Color) shr 8) and $000000ff;
      FromB := (ColorToRGB(FColors.Color) shr 16) and $000000ff;
      DoNone(FromR, FromG, FromB);

      Frame3D(bm.Canvas, rct, FBorderColors.LightColors.Color,
        FBorderColors.ShadowColors.Color, FBorderWidth);  // Draw normal frame
      if FFrameLayout = flNormal then begin
        bm.Canvas.Pen.Color := FBorderColors.DkShadowColors.Color;
        bm.Canvas.MoveTo(crct.Left, crct.Bottom-1);
        bm.Canvas.LineTo(crct.Right-1, crct.Bottom-1);
        bm.Canvas.LineTo(crct.Right-1, crct.Top-1);

        bm.Canvas.Pen.Color := FBorderColors.ShadowColors.Color;
        bm.Canvas.MoveTo(crct.Left+FBorderWidth, crct.Bottom-1-FBorderWidth);
        bm.Canvas.LineTo(crct.Right-1-FBorderWidth, crct.Bottom-1-FBorderWidth);
        bm.Canvas.LineTo(crct.Right-1-FBorderWidth, crct.Top-1+FBorderWidth);
      end;
    end;
  end
  else begin  // if Mouse down do down button
    offset := 1;            // if mouse down, we want caption to 'move' and reverse the frame
    FromR := ColorToRGB(FColors.DownColor) and $000000ff;   // Strip out separate RGB values
    FromG := (ColorToRGB(FColors.DownColor) shr 8) and $000000ff;
    FromB := (ColorToRGB(FColors.DownColor) shr 16) and $000000ff;
    DoNone(FromR, FromG, FromB);
    Frame3D(bm.Canvas, rct, FBorderColors.ShadowColors.DownColor,
      FBorderColors.LightColors.DownColor, FBorderWidth);
  end;

  if FImage <> nil then DoImage;

  Caption2 := '';
  if Length(Caption) > 0 then begin
    for i := 1 to Length(Caption)-1 do
      if (Caption[i] <> '&') or (Caption[i+1] = '&') then
        Caption2 := Caption2+Caption[i];
    Caption2 := Caption2+Caption[Length(Caption)];
  end;

  if Assigned(DropDownMenu) then
    FMarkerOffset := -6
  else
    FMarkerOffset := 0;

  // disabled button is inset
  if not Enabled then DoInset(offset)
  else
    Case FTextStyle of
      tsHeavyLowered, tsLowered: DoInset(offset);
      tsHeavyRaised, tsRaised: DoRaise(offset);
      tsNone: DoNorm(offset);
    end;

  // if we have the Focus, draw a little rectangular indicator.
  if not (csDesigning in ComponentState) then
    if FShowFocus = True then
      if Focused then
        bm.Canvas.DrawFocusRect(RECT(2,  2, Width - 2, Height - 2));

  { Finally, the button is all painted on the bitmap canvas. Now we just need
    to copy it to the canvas of our control. BitBlt is one method; there are
    several others. }
  BitBlt(Canvas.Handle, 0, 0, Width, Height, bm.Canvas.Handle, 0, 0, SRCCOPY);

  if mUp then FMenuPoppedUp := False; // if button now up, menu must have disappeared
end;

procedure TMFButton.KeyAccel(var Msg: TCMDialogChar);
begin
  with Msg do begin
    if IsAccel(CharCode, Caption) and Enabled then begin
      Click;
      Result := 1;
    end
    else
     inherited;
  end;
end;

procedure TMFButton.OnChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TMFButton.OnTextColorChanged(Sender: TObject);
begin
  Font.Color := FTextColors.Colors.Color;
  Invalidate;
end;

procedure TMFButton.ChangedImage(Sender:TObject);
begin
  Invalidate;
end;

procedure TMFButton.SetImage(Value: TBitmap);
begin
  if Assigned(Value) then
    FImage.Assign(Value)
  else begin
    FImage.Height := 0;
    FImage.Width := 0;
  end;
end;

procedure TMFButton.SetBorderWidth(Value: Integer);
begin
  if FBorderWidth <> Value then begin
    if Value > ((Width Div 2) - 5) then exit;
    if Value > ((Height Div 2) - 5) then exit;
    FBorderWidth := Value;
    Invalidate;
  end;
end;

procedure TMFButton.SetFrameLayout(Value: TMFFrameLayout);
begin
  if FFrameLayout <> Value then begin
    FFrameLayout := Value;
    Invalidate;
  end;
end;

procedure TMFButton.SetShowFocus(Value: Boolean);
begin
  if FShowFocus <> Value then
    FShowFocus := Value;
end;

procedure TMFButton.SetShowHoverColors(Value: Boolean);
begin
  if FShowHoverColors <> Value then
    FShowHoverColors := Value;
end;

procedure TMFButton.SetShowHoverFrame(Value: Boolean);
begin
  if FShowHoverFrame <> Value then
    FShowHoverFrame := Value;
end;

procedure TMFButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
   X, Y: Integer);
var
  PopupPoint: TPoint;
begin
  inherited;
  if not mUp then exit; // don't do more than once
  if Button = mbLeft then mUp := False;
  SetFocus;
  Invalidate;
  if (not FMenuPoppedUp) and Assigned(DropDownMenu) and
     (Button = mbLeft) then begin
    PopupPoint := ClientToScreen(Point(0, Height));
    FMenuPoppedUp := True;
    with DropdownMenu do
    begin
      PopupComponent := Self;
      Popup(PopupPoint.X, PopupPoint.Y);
    end;
    MouseUp(Button, Shift, X, Y);
    ControlState := ControlState - [csLButtonDown];
  end;
end;

procedure TMFButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
   X, Y: Integer);
begin
  inherited;
  if Clicking then begin
    Clicking := False;
    exit;
  end;
  if mUp then exit; // don't do more than once
  mUp := True;
  Invalidate;
end;

procedure TMFButton.Click; // Added this so I can re-draw button
begin
  Clicking := True;
  mUp := True;
  Invalidate;
  Application.ProcessMessages;
  inherited;
end;

procedure TMFButton.CMTextChanged(var Msg:TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TMFButton.CMEnabledChanged(var Msg: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TMFButton.CMDialogKey(var Msg: TCMDialogKey);
begin
  with Msg do
    if (((CharCode = VK_RETURN) and FActive) or
      ((CharCode = VK_ESCAPE) and FCancel)) and
      (KeyDataToShiftState(KeyData) = []) and CanFocus then
    begin
      Click;
      Result := 1;
    end
    else
      inherited;
end;

procedure TMFButton.CMFocusChanged(var Msg: TCMFocusChanged);
begin
  with Msg do
    if Sender is TMFButton then
      FActive := Sender = Self
    else
      FActive := FDefault;
//  SetButtonStyle(FActive); // changed MF
  Invalidate;
  inherited;
end;

procedure TMFButton.CMFontChanged(var Message: TMessage);
begin
  inherited;
  FTextColors.Colors.Color := Font.Color;
  Invalidate;
end;

procedure TMFButton.WMEraseBkgnd(var Msg:TMessage);
begin
  Msg.Result := 0;  // We intercept this message and tell Windows it processed OK.
end;

procedure TMFButton.WMSetFocus(var Msg:TWMSetFocus);
begin
  inherited;
  if csDesigning in ComponentState then exit;
  Invalidate;
end;

procedure TMFButton.WMKillFocus(var Msg:TWMKillFocus);
begin
  inherited;
  if csDesigning in ComponentState then exit;
  Invalidate;
end;

procedure TMFButton.SetDefault(Value: Boolean);
var
  Form: TCustomForm;
begin
  FDefault := Value;
  if HandleAllocated then begin
    Form := GetParentForm(Self);
    if Form <> nil then
      Form.Perform(CM_FOCUSCHANGED, 0, Longint(Form.ActiveControl));
  end;
end;

procedure TMFButton.SetAlignment(Value: TAlignment);
begin
  if FAlignment <> Value then begin
    FAlignment := Value;
    Invalidate;
  end;
end;

procedure TMFButton.SetStyle(Value: TMFTextStyle);
begin
  { When user selects a style, set the variable to new style and do
    an invalidate to force a re-paint of the component. }
  if FTextStyle <> Value then begin
    FTextStyle := Value;
    Invalidate;
  end;
end;

{ The following two procedures process user code that can be specified in the
  events for the component.  These are often used to change the color when the
  mouse enters a control, or something like that.  I made these into events so
  the user can trigger whatever code they wish without modifying/creating a
  component. }
procedure TMFButton.CMMouseEnter(var Msg:TMessage);
begin
  inherited;
  if csLButtonDown in ControlState then begin
    Self.MouseDown(mbLeft, [ssLeft], 0, 0); // Passed values are ignored
  end;
  if (not FIsMouseOver) then Invalidate;
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

procedure TMFButton.CMMouseLeave(var Msg:TMessage);
begin
  inherited;
  if csLButtonDown in ControlState then begin
    Self.MouseUp(mbLeft, [ssLeft], 0, 0);  // Passed values are ignored
  end;
  if (FIsMouseOver) then Invalidate;
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

procedure TMFButton.DoNone(fr, fg, fb: Integer);
var
  x1, y1, x2, y2: Real;
begin
  bm.Canvas.Pen.Style := psClear;
  bm.Canvas.Pen.Mode := pmCopy;
  x1 := 0;
  x2 := bm.Width+2;
  y1 := 0;
  y2 := bm.Height+2;
  bm.Canvas.Brush.Color := RGB(fr, fg, fb);   // Plug colors into brush
  bm.Canvas.FillRect(Rect(Trunc(x1), Trunc(y1), Trunc(x2), Trunc(y2)));
  bm.Canvas.Pen.Style := psSolid;
end;

procedure TMFButton.DoImage;
//var
//  x1, y1, x2, y2: Real;
begin
  bm.Canvas.Pen.Style := psClear;
  bm.Canvas.Pen.Mode := pmCopy;
//  x1 := 0;
//  x2 := bm.Width+2;
//  y1 := 0;
//  y2 := bm.Height+2;

  // paint the image
//  bm.Canvas.Brush.Color := clBlack;
//  bm.Canvas.FrameRect(Rect(Trunc(x1), Trunc(y1), Trunc(x2), Trunc(y2)));
  bm.Canvas.Draw((bm.Width-FImage.Width) div 2, (bm.Height-FImage.Height) div 2,
    FImage);
  bm.Canvas.Pen.Style := psSolid;
end;

procedure TMFButton.DoRaise(offset: Integer);
{This draws 'raised' 3D text for the Caption.  Inset is the same principal,
 the colors are just reversed.  The brush style is set to Clear so the
 background is not erased.  The highlight colored text is drawn on the canvas
 at -1 up and left of center.  if heavy shading is set, the shadow colored
 text is drawn at +1 right and down of center.  Finally, the Font.Color colored
 text is drawn in exact center.  The offset parameter is used to 'move' the
 whole shebang down and right 1 if the Mouse is Down.}
var
  r: TRect;
  J: integer;    // For justification
begin
  bm.Canvas.Brush.Style := bsClear;
  if mUp then
    if FIsMouseOver and ShowHoverColors then
      bm.Canvas.Font.Color := FTextColors.LightColors.HoverColor
    else
      bm.Canvas.Font.Color := FTextColors.LightColors.Color
  else
      bm.Canvas.Font.Color := FTextColors.LightColors.DownColor;
  J := DT_CENTER;    // Initialize to avoid warnings
  case FAlignment of
    taLeftJustify   : begin
                J := DT_LEFT;
                r := Rect(1+offset,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)-1+offset,
                  0, 0);
              end;
    taCenter : begin
                J := DT_CENTER;
                r := Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) div 2)-1+offset+FMarkerOffset,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)-1+offset,
                  0, 0);
              end;
    taRightJustify  : begin
                J := DT_RIGHT;
                r := Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) - 2)-1+offset+FMarkerOffset-6,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)-1+offset,
                  0, 0);
              end;
  end;

  DrawText(bm.Canvas.Handle, PChar(Caption2), -1, r, DT_CALCRECT+J+DT_TOP);
  DrawText(bm.Canvas.Handle, PChar(Caption), -1, r, J+DT_TOP);

  if FTextStyle = tsHeavyRaised then begin
    if mUp then
      if FIsMouseOver and ShowHoverColors then
        bm.Canvas.Font.Color := FTextColors.ShadowColors.HoverColor
      else
        bm.Canvas.Font.Color := FTextColors.ShadowColors.Color
    else
        bm.Canvas.Font.Color := FTextColors.ShadowColors.DownColor;

    case FAlignment of
      taLeftJustify   : begin
                  J := DT_LEFT;
                  r := Rect(3+offset,
                    ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+1+offset,
                    0, 0);
                end;
      taCenter : begin
                  J := DT_CENTER;
                  r := Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) div 2)+1+offset+FMarkerOffset,
                    ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+1+offset,
                    0, 0);
                end;
      taRightJustify  : begin
                  J := DT_RIGHT;
                  r := Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) - 1)+offset+FMarkerOffset-6,
                    ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+1+offset,
                    0, 0);
                end;
    end;

    DrawText(bm.Canvas.Handle, PChar(Caption2), -1, r,
      DT_CALCRECT+DT_LEFT+DT_TOP);
    DrawText(bm.Canvas.Handle, PChar(Caption), -1, r, DT_CENTER+DT_TOP);
  end;

  if mUp then
    if FIsMouseOver and ShowHoverColors then
      bm.Canvas.Font.Color := FTextColors.Colors.HoverColor
    else
      bm.Canvas.Font.Color := FTextColors.Colors.Color
  else
      bm.Canvas.Font.Color := FTextColors.Colors.DownColor;
  case FAlignment of
    taLeftJustify   : begin
                J := DT_LEFT;
                r := Rect(2+offset,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,
                  0, 0);
              end;
    taCenter : begin
                J := DT_CENTER;
                r := Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) div 2)+offset+FMarkerOffset,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,
                  0, 0);
              end;
    taRightJustify  : begin
                J := DT_RIGHT;
                r := Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) - 2)+offset+FMarkerOffset-6,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,
                  0, 0);
              end;
  end;

  DrawText(bm.Canvas.Handle, PChar(Caption2), -1, r, DT_CALCRECT+J+DT_TOP);
  DrawText(bm.Canvas.Handle, PChar(Caption), -1, r, J+DT_TOP);
  bm.Canvas.Brush.Style := bsSolid;
end;

procedure TMFButton.DoInset(offset: Integer);
var
  r: TRect;
  J: integer;         // For justification
begin
  bm.Canvas.Brush.Style := bsClear;
  J := DT_CENTER;

  if FTextStyle = tsHeavyLowered then begin
    if mUp then
      if FIsMouseOver and ShowHoverColors then
        bm.Canvas.Font.Color := FTextColors.ShadowColors.HoverColor
      else
        bm.Canvas.Font.Color := FTextColors.ShadowColors.Color
    else
        bm.Canvas.Font.Color := FTextColors.ShadowColors.DownColor;

    case FAlignment of
      taLeftJustify   : begin
                  J := DT_LEFT;
                  r := Rect(1+offset,
                    ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)-1+offset,
                    0, 0);
                end;
      taCenter : begin
                  J := DT_CENTER;
                  r := Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) div 2)-1+offset+FMarkerOffset,
                    ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)-1+offset,
                    0, 0);
                end;
      taRightJustify  : begin
                  J := DT_RIGHT;
                  r := Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) - 2)-1+offset+FMarkerOffset-6,
                    ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)-1+offset,
                    0, 0);
                end;
    end;

    DrawText(bm.Canvas.Handle, PChar(Caption2), -1, r, DT_CALCRECT+J+DT_TOP);
    DrawText(bm.Canvas.Handle, PChar(Caption), -1, r, J+DT_TOP);
  end;

  if mUp then
    if FIsMouseOver and ShowHoverColors then
      bm.Canvas.Font.Color := FTextColors.LightColors.HoverColor
    else
      bm.Canvas.Font.Color := FTextColors.LightColors.Color
  else
      bm.Canvas.Font.Color := FTextColors.LightColors.DownColor;

  case FAlignment of
    taLeftJustify   : begin
                J := DT_LEFT;
                r := Rect(3+offset,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+1+offset,
                  0, 0);
              end;
    taCenter : begin
                J := DT_CENTER;
                r := Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) div 2)+1+offset+FMarkerOffset,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+1+offset,
                  0, 0);
              end;
    taRightJustify  : begin
                J := DT_RIGHT;
                r := Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) - 1)+offset+FMarkerOffset-6,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+1+offset,
                  0, 0);
              end;
  end;

  DrawText(bm.Canvas.Handle, PChar(Caption2), -1, r, DT_CALCRECT+J+DT_TOP);
  DrawText(bm.Canvas.Handle, PChar(Caption), -1, r, J+DT_TOP);

  if Enabled then
    if mUp then
      if FIsMouseOver and ShowHoverColors then
        bm.Canvas.Font.Color := FTextColors.Colors.HoverColor
      else
        bm.Canvas.Font.Color := FTextColors.Colors.Color
    else
        bm.Canvas.Font.Color := FTextColors.Colors.DownColor
  else
    bm.Canvas.Font.Color := FTextColors.DisabledColor;

  case FAlignment of
    taLeftJustify   : begin
                J := DT_LEFT;
                r := Rect(2+offset,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset, 0, 0);
              end;
    taCenter : begin
                J := DT_CENTER;
                r := Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) div 2)+offset+FMarkerOffset,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,
                  0, 0);
              end;
    taRightJustify  : begin
                J := DT_RIGHT;
                r := Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) - 2)+offset+FMarkerOffset-6,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,
                  0, 0);
              end;
  end;

  DrawText(bm.Canvas.Handle, PChar(Caption2), -1, r, DT_CALCRECT+J+DT_TOP);
  DrawText(bm.Canvas.Handle, PChar(Caption), -1, r, J+DT_TOP);
  bm.Canvas.Brush.Style := bsSolid;
end;

procedure TMFButton.DoNorm(offset: Integer);
var
  r: TRect;
  J: integer;  // For justification
begin           // if no 3D effect is set, we just draw text
  bm.Canvas.Brush.Style := bsClear;
  J := DT_CENTER;     // Initialized to avoid warnings

  if mUp then
    if FIsMouseOver and ShowHoverColors then
      bm.Canvas.Font.Color := FTextColors.Colors.HoverColor
    else
      bm.Canvas.Font.Color := FTextColors.Colors.Color
  else
      bm.Canvas.Font.Color := FTextColors.Colors.DownColor;

  case FAlignment of
    taLeftJustify   : begin
                J := DT_LEFT;
                r := Rect(2+offset,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,
                  0, 0);
              end;
    taCenter : begin
                J := DT_CENTER;
                r := Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) div 2)+offset+FMarkerOffset,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,
                  0, 0);
              end;
    taRightJustify  : begin
                J := DT_RIGHT;
                r := Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) - 2)+offset+FMarkerOffset-6,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,
                  0, 0);
              end;
  end;

  DrawText(bm.Canvas.Handle, PChar(Caption2), -1, r, DT_CALCRECT+J+DT_TOP);
  DrawText(bm.Canvas.Handle, PChar(Caption), -1, r, J+DT_TOP);
  bm.Canvas.Brush.Style := bsSolid;
end;

{I added the following to allow the Enter key to behave as a Click.
if return key was pressed, do a 'Click'. This only happens if we have the 'Focus'.}
procedure TMFButton.KeyPress(var Key: Char);
begin
  inherited;
  if Key = Chr(13) then Click;
end;

procedure TMFButton.DrawPopupMark;
var
  y: Integer;
begin
  y := (Height - 4) div 2;
  with bm.Canvas do begin
    Pen.Color := FTextColors.Colors.Color;
    Brush.Color := FTextColors.Colors.Color;
    Brush.Style := bsSolid;
    Polygon([Point(Width - 14, y), Point(Width - 8, y),
      Point(Width - 11, y + 3)]);
  end;
end;

procedure TMFButton.SetDropdownMenu(Value: TPopupMenu);
begin
  if FDropdownMenu <> Value then begin
    FDropdownMenu := Value;
    if Assigned(Value) then
      Value.FreeNotification(Self);
    Invalidate;
  end;
end;

procedure TMFButton.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = DropdownMenu) then
    DropdownMenu := nil;
end;

procedure TMFButton.SetEnabled(value: Boolean);
begin
  inherited;
  Invalidate;
end;

procedure TMFButton.SetBorderColors(Value: TMFBorderColors);
begin
  inherited;
  FBorderColors.Assign(Value);
  Invalidate;
end;

procedure TMFButton.SetTextColors(Value: TMFTextColors);
begin
  inherited;
  FTextColors.Assign(Value);
  Font.Color := Value.Colors.Color;
  Invalidate;
end;

procedure TMFButton.SetColors(Value: TMFMouseColors);
begin
  inherited;
  FColors.Assign(Value);
  Invalidate;
end;

{ - TMFGradButton ------------------------------------------------------------ }

constructor TMFGradButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FGradient := TMFGradient.Create;
  Gradient.OnChanged := OnChanged;

  mUp := True;                    // Mouse is UP to start with
  FSwapColor := True;             // By default, we swap colors on Mouse Down
  FPreset := None;
end;

destructor TMFGradButton.Destroy;
begin
  FGradient.Free;
  inherited Destroy;
end;

procedure TMFGradButton.SetGradient(Value: TMFGradient);
begin
  FGradient.Assign(Value);
  Invalidate;
end;

function TMFGradButton.Muldv(a, b, c: integer): longint;
ASM
  MOV EAX, a
  IMUL b
  IDIV c
end;

{Everything pretty much happens in the Paint procedure; a lot of it is farmed
 out to other procedures just so the logic in the paint procedure can be
 understood a little better.  Here's the basic process for creating a gradient:
 Get the separate R, G, and B values of the BeginClr Color.  then, find the
 difference between that and the R, G, and B values of the EndClr Color.  Using
 these two sets of values, draw a series of 'shapes' using a series of colors
 between the begin and end.  After the gradient is drawn, add a frame and the
 Caption.}
procedure TMFGradButton.Paint;
var
  FromR, FromG, FromB: Integer; // These are the separate color values for RGB
  DiffR, DiffG, DiffB: Integer; // of color values.
  CursorPos: TPoint;
  rct, crct: TRect;             // Rectangle used to draw frame around button
  offset: Integer;              // Used for Caption location during Mouse Down
  i: Integer; 			 // loop variable to iterate Caption for &. ABB
{ To speed things up and reduce flicker, I use a Bitmap to draw the button in
  its entirety, then BitBlt it to the canvas of the control. }
begin
  //inherited;                  // PROBABLY DON'T NEED THIS
  bm.Canvas.Font := Font;       // Set the Bitmap font to match control font
  bm.Width := Width;            // Set BMP dimensions to match control's
  bm.Height := Height;
  rct := Rect(0, 0, Width, Height);      // Set rectangle size for later use

  FromR := ColorToRGB(FGradient.BeginColor) and $000000ff;           // Strip out separate RGB values
  FromG := (ColorToRGB(FGradient.BeginColor) shr 8) and $000000ff;
  FromB := (ColorToRGB(FGradient.BeginColor) shr 16) and $000000ff;
  DiffR := (ColorToRGB(FGradient.EndColor) and $000000ff) - FromR;   // Find the difference
  DiffG := ((ColorToRGB(FGradient.EndColor) shr 8) and $000000ff) - FromG;
  DiffB := ((ColorToRGB(FGradient.EndColor) shr 16) and $000000ff) - FromB;
  // Depending on gradient style selected, go draw it on the Bitmap canvas.
  case FGradient.Style of
    gsHorizontal  : DoHorizontal(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsVertical    : DoVertical(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsElliptic    : DoElliptic(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsRectangle   : DoRectangle(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsVertCenter  : DoVertCenter(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsHorizCenter : DoHorizCenter(FromR, FromG, FromB, DiffR, DiffG, DiffB);
  end;

  if Assigned(DropDownMenu) then DrawPopupMark;

  // By setting the Brush style to Clear, it will draw without overlaying bkgrnd
  bm.Canvas.Brush.Style := bsClear;  // Gradient is done, time for Hilite-Shadow

  if mUp = True then begin  // if Mouse up do normal (or hover) button
    offset := 0;            // offset is used to move the Caption down a notch when the button is pressed.

    crct.TopLeft := Parent.ClientToScreen(Point(Left, Top));
    crct.BottomRight := Parent.ClientToScreen(Point(Left + Width, Top + Height));
    GetCursorPos(CursorPos);
    FIsMouseOver := PtInRect(crct, CursorPos);

    crct := GetClientRect;
    if FIsMouseOver and FShowHoverColors then begin
      if FGradient.Style = gsNone then begin
        FromR := ColorToRGB(FColors.HoverColor) and $000000ff;   // Strip out separate RGB values
        FromG := (ColorToRGB(FColors.HoverColor) shr 8) and $000000ff;
        FromB := (ColorToRGB(FColors.HoverColor) shr 16) and $000000ff;
        DoNone(FromR, FromG, FromB);
      end;

      if (FShowHoverFrame and (FrameLayout = flFlat)) or
         (FrameLayout = flNormal) then begin
        Frame3D(bm.Canvas, rct, FBorderColors.LightColors.HoverColor,
          FBorderColors.DkShadowColors.HoverColor, FBorderWidth);  // Draw mousehover frame
        bm.Canvas.Pen.Color := FBorderColors.ShadowColors.HoverColor;
        bm.Canvas.MoveTo(crct.Left+FBorderWidth, crct.Bottom-1-FBorderWidth);
        bm.Canvas.LineTo(crct.Right-1-FBorderWidth, crct.Bottom-1-FBorderWidth);
        bm.Canvas.LineTo(crct.Right-1-FBorderWidth, crct.Top-1+FBorderWidth);
      end
      else
        Frame3D(bm.Canvas, rct, FBorderColors.LightColors.HoverColor,
          FBorderColors.ShadowColors.HoverColor, FBorderWidth);  // Draw mousehover frame
    end
    else begin
      if FGradient.Style = gsNone then begin
        FromR := ColorToRGB(FColors.Color) and $000000ff;   // Strip out separate RGB values
        FromG := (ColorToRGB(FColors.Color) shr 8) and $000000ff;
        FromB := (ColorToRGB(FColors.Color) shr 16) and $000000ff;
        DoNone(FromR, FromG, FromB);
      end;

      Frame3D(bm.Canvas, rct, FBorderColors.LightColors.Color,
        FBorderColors.ShadowColors.Color, FBorderWidth);  // Draw normal frame
      if FFrameLayout = flNormal then begin
        bm.Canvas.Pen.Color := FBorderColors.DkShadowColors.Color;
        bm.Canvas.MoveTo(crct.Left, crct.Bottom-1);
        bm.Canvas.LineTo(crct.Right-1, crct.Bottom-1);
        bm.Canvas.LineTo(crct.Right-1, crct.Top-1);

        bm.Canvas.Pen.Color := FBorderColors.ShadowColors.Color;
        bm.Canvas.MoveTo(crct.Left+FBorderWidth, crct.Bottom-1-FBorderWidth);
        bm.Canvas.LineTo(crct.Right-1-FBorderWidth, crct.Bottom-1-FBorderWidth);
        bm.Canvas.LineTo(crct.Right-1-FBorderWidth, crct.Top-1+FBorderWidth);
      end;
    end;
  end
  else begin  // if Mouse down do down button
    offset := 1;            // if mouse down, we want caption to 'move' and reverse the frame
    if FGradient.Style = gsNone then begin
      FromR := ColorToRGB(FColors.DownColor) and $000000ff;   // Strip out separate RGB values
      FromG := (ColorToRGB(FColors.DownColor) shr 8) and $000000ff;
      FromB := (ColorToRGB(FColors.DownColor) shr 16) and $000000ff;
      DoNone(FromR, FromG, FromB);
    end;
    Frame3D(bm.Canvas, rct, FBorderColors.ShadowColors.DownColor,
      FBorderColors.LightColors.DownColor, FBorderWidth);
  end;

  if FImage <> nil then DoImage;

  Caption2 := '';
  if Length(Caption) > 0 then begin
    for i := 1 to Length(Caption)-1 do
      if (Caption[i] <> '&') or (Caption[i+1] = '&') then
        Caption2 := Caption2+Caption[i];
    Caption2 := Caption2+Caption[Length(Caption)];
  end;

  if Assigned(DropDownMenu) then
    FMarkerOffset := -6
  else
    FMarkerOffset := 0;

  // disabled button is inset
  if not Enabled then DoInset(offset)
  else
    Case FTextStyle of
      tsHeavyLowered, tsLowered: DoInset(offset);
      tsHeavyRaised, tsRaised: DoRaise(offset);
      tsNone: DoNorm(offset);
    end;

  // if we have the Focus, draw a little rectangular indicator.
  if not (csDesigning in ComponentState) then
    if FShowFocus = True then
      if Focused then
        bm.Canvas.DrawFocusRect(RECT(2,  2, Width - 2, Height - 2));

  { Finally, the button is all painted on the bitmap canvas.  Now we just need
    to copy it to the canvas of our control.  BitBlt is one method; there are
    several others. }
  BitBlt(Canvas.Handle, 0, 0, Width, Height, bm.Canvas.Handle, 0, 0, SRCCOPY);

  if mUp then FMenuPoppedUp := False; // if button now up, menu must have disappeared
end;

procedure TMFGradButton.SetSwap(Value: Boolean);
begin
  if FSwapColor <> Value then
    FSwapColor := Value;
end;

procedure TMFGradButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer);
var
  PopupPoint: TPoint;
begin
//  inherited;
  if not mUp then exit; // don't do more than once
  if Button = mbLeft then mUp := False;
  SetFocus;
  if FSwapColor = True then begin    // if SwapClrs is true, we need to save the original stuff.  Hence, TmpClr
    if not (FGradient.Style = gsNone) then begin
      TmpClr := FGradient.BeginColor;
      FGradient.BeginColor := FGradient.EndColor;
      FGradient.EndColor := TmpClr;
    end;
  end;
  Invalidate;
  if (not FMenuPoppedUp) and Assigned(DropDownMenu) and
     (Button = mbLeft) then begin
    PopupPoint := ClientToScreen(Point(0, Height));
    FMenuPoppedUp := True;
    with DropdownMenu do
    begin
      PopupComponent := Self;
      Popup(PopupPoint.X, PopupPoint.Y);
    end;
    MouseUp(Button, Shift, X, Y);
    ControlState := ControlState - [csLButtonDown];
  end;
end;

procedure TMFGradButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer);
begin
  inherited;
  if Clicking then begin
    Clicking := False;
    exit;
  end;
  if mUp then exit; // don't do more than once
  if (FSwapColor = True) and (mUp = False) and
     not (FGradient.Style = gsNone) then begin
    TmpClr := FGradient.BeginColor;
    FGradient.BeginColor := FGradient.EndColor;
    FGradient.EndColor := TmpClr;
  end;
  mUp := True;
  Invalidate;
end;

procedure TMFGradButton.Click; // Added this so I can re-draw button
begin
  Clicking := True;
  if (FSwapColor = True) and not (FGradient.Style = gsNone) then begin
    TmpClr := FGradient.BeginColor;
    FGradient.BeginColor := FGradient.EndColor;
    FGradient.EndColor := TmpClr;
  end;
  mUp := True;
  Invalidate;
  Application.ProcessMessages;
  inherited;
end;

procedure TMFGradButton.SetPreset(Value: TMFPreset);
begin
  if csDesigning in ComponentState then begin
    case Value of
      None : begin
(*
        FGradient.FBeginColor := clNavy;
        FGradient.FEndColor := clAqua;
        Font.Size := 8;
        Font.Name := 'MS Sans Serif';
        Font.Color := clBlack;
        Font.Style := [];
        FGradient.FStyle := gsVertCenter;
        FTextStyle := tsRaised;
        FTextColors.FLightColor := clBtnHighLight;
        FTextColors.FShadowColor := clBtnShadow;
        FBorderColors.LightColor := clBtnHighlight;
        FBorderColors.ShadowColor := clBtnShadow;
*)
      end;
      GoldCoast : begin
        FTextColors.DisabledColor := $0000bbbb;
        FGradient.BeginColor := clYellow;
        FGradient.EndColor := clOlive;
        Font.Size := 10;
        Font.Name := 'Arial';
        Font.Color := $0000bbbb;
        Font.Style := [fsBold];
        FGradient.Style := gsVertical;
        FTextStyle := tsHeavyRaised;
        FTextColors.LightColors.Color := $00bfffff;
        FTextColors.ShadowColors.Color := $00007171;
        FBorderColors.LightColors.Color := $00b3ffff;
        FBorderColors.ShadowColors.Color := $00616161;
      end;
      LimeBox : begin
        FTextColors.DisabledColor := clOlive;
        FGradient.BeginColor := clBlack;
        FGradient.EndColor := clLime;
        Font.Size := 8;
        Font.Name := 'Courier';
        Font.Color := clGreen;
        Font.Style := [];
        FGradient.Style := gsRectangle;
        FTextStyle := tsRaised;
        FTextColors.LightColors.Color := $00acffac;
        FTextColors.ShadowColors.Color := $00004000;
        FBorderColors.LightColors.Color := clBtnHighLight;
        FBorderColors.ShadowColors.Color := clBtnShadow;
      end;
      Aluminum : begin
        FTextColors.DisabledColor := clGray;
        FGradient.BeginColor := $00cecece;
        FGradient.EndColor := $005f5f5f;
        Font.Size := 10;
        Font.Name := 'Times New Roman';
        Font.Color := clBlack;
        Font.Style := [fsBold];
        FGradient.Style := gsVertical;
        FTextStyle := tsHeavyLowered;
        FTextColors.LightColors.Color := clBtnHighlight;
        FTextColors.ShadowColors.Color := clBtnShadow;
        FBorderColors.LightColors.Color := clWhite;
        FBorderColors.ShadowColors.Color := clBlack;
      end;
      SunRise : begin
        FTextColors.DisabledColor := clGray;
        FGradient.BeginColor := clYellow;
        FGradient.EndColor := clRed;
        Font.Size := 8;
        Font.Name := 'MS Sans Serif';
        Font.Color := $00000040;
        Font.Style := [fsBold];
        FGradient.Style := gsHorizontal;
        FTextStyle := tsHeavyRaised;
        FTextColors.LightColors.Color := $00cfe7e6;
        FTextColors.ShadowColors.Color := $000058b0;
        FBorderColors.LightColors.Color := clBtnHighlight;
        FBorderColors.ShadowColors.Color := clBtnShadow;
      end;
      BrassedOff : begin
        FTextColors.DisabledColor := clOlive;
        FGradient.BeginColor := $00CEFFFF;
        FGradient.EndColor := $0092C2C2;
        Font.Size := 9;
        Font.Name := 'Verdana';
        Font.Color := clMaroon;
        Font.Style := [];
        FGradient.Style := gsElliptic;
        FTextStyle := tsNone;
        FSwapColor := False;
        FTextColors.LightColors.Color := $00EFFFFF;
        FTextColors.ShadowColors.Color := $0092C2C2;
        FBorderColors.LightColors.Color := $00EFFFFF;
        FBorderColors.ShadowColors.Color := $0092C2C2;
      end;
      Encarta : begin
        Height := 22;
        Width := 86;
        FTextColors.DisabledColor := clBtnFace;
        FColors.Color := clBtnFace;
        FColors.HoverColor := $000ABED8;
        FColors.DownColor := $000ABED8;
        Font.Size := 8;
        Font.Name := 'Arial';
        Font.Color := clBlack;
        Font.Style := [];
        FGradient.Style := gsNone;
        FTextStyle := tsNone;
        FFrameLayout := flFlat;
        FShowHoverColors := True;
        FShowHoverFrame := False;
        FBorderColors.LightColors.Color := $00828F99;
        FBorderColors.ShadowColors.Color := $00828F99;
        FBorderColors.LightColors.HoverColor := $00EDF4F8;
        FBorderColors.ShadowColors.HoverColor := $004F4F4F;
      end;
      Select : begin
      end;
      Aol : begin
      end;
      else
        Exit;
    end;
    FPreset := Value;
    Invalidate;
  end;
end;

{I'll explain a little about the Horizontal gradient, the other styles are all
 consistent with their logic.  The six R, G, and B values are passed to us.
 We define some local variables we'll need: a rectangle, a FOR loop counter,
 and our own RGB numbers.  For a horizontal gradient, we'll draw a series of
 rectangles, each one a little closer in color to the EndClr value.  A horizontal
 gradient rectangle will always be from the top to the bottom of the canvas,
 so we set top to 0 and bottom to however tall our control is.  then, we draw
 a series of 255 rectangles.  The starting point and width of each will depend
 on the actual width of our control.  It starts out on the left, draws the
 first rectangle in a color that's a percentage of the difference plus the
 starting color.  As I increments through the loop, the rectangles move to the
 right and the color gets closer and closer to the EndClr.}
procedure TMFGradButton.DoHorizontal(fr, fg, fb, dr, dg, db: Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B: Byte;
begin
  ColorRect.Top := 0;                 // Set rectangle top
  ColorRect.Bottom := Height;
  for I := 0 to 255 do begin         // Make lines (rectangles) of color
    ColorRect.Left := Muldv(I, Width, 256);       // Find left for this color
    ColorRect.Right := Muldv(I + 1, Width, 256);  // Find Right
    R := fr + Muldv(I, dr, 255);     // Find the RGB values
    G := fg + Muldv(I, dg, 255);
    B := fb + Muldv(I, db, 255);
    bm.Canvas.Brush.Color := RGB(R, G, B);   // Plug colors into brush
    bm.Canvas.FillRect(ColorRect);           // Draw on Bitmap
  end;
end;

procedure TMFGradButton.DoVertical(fr, fg, fb, dr, dg, db: Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B: Byte;
begin
  ColorRect.Left := 0;                // Set rectangle left&right
  ColorRect.Right := Width;
  for I := 0 to 255 do begin         // Make lines (rectangles) of color
    ColorRect.Top := Muldv(I, Height, 256);         // Find top for this color
    ColorRect.Bottom := Muldv(I + 1, Height, 256);  // Find Bottom
    R := fr + Muldv(I, dr, 255);     // Find the RGB values
    G := fg + Muldv(I, dg, 255);
    B := fb + Muldv(I, db, 255);
    bm.Canvas.Brush.Color := RGB(R, G, B);   // Plug colors into brush
    bm.Canvas.FillRect(ColorRect);           // Draw on Bitmap
  end;
end;

procedure TMFGradButton.DoElliptic(fr, fg, fb, dr, dg, db: Integer);
var
  I: Integer;
  R, G, B: Byte;
  Pw, Ph, Dw, Dh: integer;
  x1, y1, x2, y2: integer;
{The elliptic is a bit different, since I had to use real numbers. I cut down
 on the number (to 155 instead of 255) of iterations in an attempt to speed
 things up, to no avail.  I think it just takes longer for windows to draw an
 ellipse as opposed to a rectangle.}
begin
  bm.Canvas.Pen.Style := psClear;
  bm.Canvas.Pen.Mode := pmCopy;
  x1 := bm.Width div -3;
  x2 := bm.Width + (bm.Width div 3);
  y1 := bm.Height div -3;
  y2 := bm.Height + (bm.Height div 3);
  Pw := x2 - x1;
  Ph := y2 - y1;
  for I := 0 to 50 do begin         // Make ellipses of color
    R := fr + Muldv(I, dr, 50);     // Find the RGB values
    G := fg + Muldv(I, dg, 50);
    B := fb + Muldv(I, db, 50);
    bm.Canvas.Brush.Color := R or (G shl 8) or (b shl 16);  // Plug colors into brush
    Dw := Pw * i div 100;
    Dh := Ph * i div 100;
    bm.Canvas.Ellipse(x1 + Dw, y1 + Dh, x2 - Dw, y2 - Dh);
  end;
  bm.Canvas.Pen.Style := psSolid;
end;

procedure TMFGradButton.DoRectangle(fr, fg, fb, dr, dg, db: Integer);
var
  I: Integer;
  R, G, B: Byte;
  Pw, Ph : Real;
  x1, y1, x2, y2: Real;
begin
  bm.Canvas.Pen.Style := psClear;
  bm.Canvas.Pen.Mode := pmCopy;
  x1 := 0;
  x2 := bm.Width+2;
  y1 := 0;
  y2 := bm.Height+2;
  Pw := (bm.Width / 2) / 255;
  Ph := (bm.Height / 2) / 255;
  for I := 0 to 255 do begin         // Make rectangles of color
    x1 := x1 + Pw;
    x2 := X2 - Pw;
    y1 := y1 + Ph;
    y2 := y2 - Ph;
    R := fr + Muldv(I, dr, 255);     // Find the RGB values
    G := fg + Muldv(I, dg, 255);
    B := fb + Muldv(I, db, 255);
    bm.Canvas.Brush.Color := RGB(R, G, B);   // Plug colors into brush
    bm.Canvas.FillRect(Rect(Trunc(x1), Trunc(y1), Trunc(x2), Trunc(y2)));
  end;
  bm.Canvas.Pen.Style := psSolid;
end;

procedure TMFGradButton.DoVertCenter(fr, fg, fb, dr, dg, db: Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B: Byte;
  Haf: Integer;
begin
  Haf := bm.Height Div 2;
  ColorRect.Left := 0;
  ColorRect.Right := bm.Width;
  for I := 0 to Haf do begin
    ColorRect.Top := Muldv(I, Haf, Haf);
    ColorRect.Bottom := Muldv(I + 1, Haf, Haf);
    R := fr + Muldv(I, dr, Haf);
    G := fg + Muldv(I, dg, Haf);
    B := fb + Muldv(I, db, Haf);
    bm.Canvas.Brush.Color := RGB(R, G, B);
    bm.Canvas.FillRect(ColorRect);
    ColorRect.Top := bm.Height - (Muldv(I, Haf, Haf));
    ColorRect.Bottom := bm.Height - (Muldv(I + 1, Haf, Haf));
    bm.Canvas.FillRect(ColorRect);
  end;
end;

procedure TMFGradButton.DoHorizCenter(fr, fg, fb, dr, dg, db: Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B: Byte;
  Haf: Integer;
begin
  Haf := bm.Width Div 2;
  ColorRect.Top := 0;
  ColorRect.Bottom := bm.Height;
  for I := 0 to Haf do begin
    ColorRect.Left := Muldv(I, Haf, Haf);
    ColorRect.Right := Muldv(I + 1, Haf, Haf);
    R := fr + Muldv(I, dr, Haf);
    G := fg + Muldv(I, dg, Haf);
    B := fb + Muldv(I, db, Haf);
    bm.Canvas.Brush.Color := RGB(R, G, B);
    bm.Canvas.FillRect(ColorRect);
    ColorRect.Left := bm.Width - (Muldv(I, Haf, Haf));
    ColorRect.Right := bm.Width - (Muldv(I + 1, Haf, Haf));
    bm.Canvas.FillRect(ColorRect);
  end;
end;

end.
