{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

Initial Autor   : Marcus Fuchs

Last Modified at: 04. February 2003
Last Modified by: Marcus Fuchs

You may retrieve the latest version of this file at the MFLib home page,
located at http://www.mfhome.net/mflib/
-------------------------------------------------------------------------------}

unit MFDialogs;

interface { ------------------------------------------------------ interface - }

// remove the period to enable Debug settings (see MFLib.inc)
{.$DEFINE DEBUG}

{$I MFLib.inc}

uses
  Classes, Controls, Forms, Windows, SysUtils,                         { VCL   }
  MFTypes;                                                             { MFLib }

type
  TMFSearchDialog = class(TComponent)
  private
    FStyle: TFindDlgStyle;
    FRange: TSearchRange;
    FOptions: TRichSearchTypes;
  protected
  public
    function Execute(var Find: String): TModalResult;
    constructor Create(AOwner: TComponent); override;
  published
    property Style: TFindDlgStyle read FStyle write FStyle default dsEmpty;
    property Range: TSearchRange read FRange write FRange default srWholeTxt;
    property Options: TRichSearchTypes read FOptions write FOptions default [];
  end;

type
  TMFReplaceDialog = class(TComponent)
  private
    FStyle: TFindDlgStyle;
    FRange: TSearchRange;
    FOptions: TRichSearchTypes;
  protected
  public
    function Execute(var Find: String; var Replace: String): TModalResult;
    constructor Create(AOwner: TComponent); override;
  published
    property Style: TFindDlgStyle read FStyle write FStyle default dsEmpty;
    property Range: TSearchRange read FRange write FRange default srWholeTxt;
    property Options: TRichSearchTypes read FOptions write FOptions default [];
  end;

type
  TMFAboutDialog = class(TComponent)
  private
    FHasInfo: Boolean;
    FVersionList: TStringList;

    FTitle: String;
    FProduct, FVersion, FCopyright: String;
    FHomepage: String;
    FEmail: String;
    FInfo: String;
    FStyle: TMFAboutDlgStyle;
  protected
    function ReadProduct: String;
    function ReadVersion: String;
    function ReadCopyright: String;
    function ReadHomepage: String;
    function ReadEmail: String;
    function ReadInfo: String;
  public
    procedure Execute;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Title: String read FTitle write FTitle;
    property Homepage: String read FHomepage write FHomepage;
    { Note: This could also be set via Project|Options, simply add a new key
      called 'Homepage'. }
    property Email: String read FEmail write FEmail;
    { Note: This could also be set via Project|Options, simply add a new key
      called 'E-mail'. }
    property Info: String read FInfo write FInfo;
    { Note: This could also be set via Project|Options, simply write somethink
      to the 'Comment'-key. }
    property DialogStyle: TMFAboutDlgStyle read FStyle write FStyle default dsModern;
  end;

function MFInputQuery(const Caption, Prompt: String; var Text: String): Boolean;
function MFSelectQuery(const Caption, Prompt: String; const List: TStrings;
   const DefIndex: Integer; var Index: Integer): Boolean;


implementation { -------------------------------------------- implementation - }

{$R ..\Resources\MFDialogs.dcr}

uses
  MFSearch, MFReplace, MFAbout, MFSelectQuery, MFInputQuery,
  MFRoutines;

var
  SearchDlg: TMFSearchDlg;
  ReplaceDlg: TMFReplaceDlg;
  AboutDlg: TMFAboutDlg;

{ - TSearchDialog ------------------------------------------------------------ }
constructor TMFSearchDialog.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FStyle := dsEmpty;
  FRange := srWholeTxt;
  FOptions := []
end;

function TMFSearchDialog.Execute(var Find: String): TModalResult;
begin
  SearchDlg := TMFSearchDlg.Create(Application);
  try
    SearchDlg.Style := FStyle;
    SearchDlg.Range := FRange;
    SearchDlg.Options := FOptions;
    SearchDlg.FindStrEdit.Text := Find;

    Result := SearchDlg.ShowModal;

    Range := SearchDlg.Range;
    Options := SearchDlg.Options;
    Find := SearchDlg.FindStrEdit.Text;
  finally
    SearchDlg.Free;
  end;
end;

{ - TReplaceDialog ----------------------------------------------------------- }
constructor TMFReplaceDialog.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FStyle := dsEmpty;
  FRange := srWholeTxt;
  FOptions := []
end;

function TMFReplaceDialog.Execute(var Find: String;
   var Replace: String): TModalResult;
begin
  ReplaceDlg := TMFReplaceDlg.Create(Application);
  try
    ReplaceDlg.Style := FStyle;
    ReplaceDlg.Range := FRange;
    ReplaceDlg.Options := FOptions;
    ReplaceDlg.FindStrEdit.Text := Find;
    ReplaceDlg.ReplaceStrEdit.Text := Replace;

    Result := ReplaceDlg.ShowModal;

    Range := ReplaceDlg.Range;
    Options := ReplaceDlg.Options;
    Find := ReplaceDlg.FindStrEdit.Text;
    Replace := ReplaceDlg.ReplaceStrEdit.Text;
  finally
    ReplaceDlg.Free;
  end;
end;

{ - TAboutDialog ------------------------------------------------------------- }
constructor TMFAboutDialog.Create(AOwner: TComponent);
begin
  inherited;

  FHasInfo := True;
  FVersionList:= TStringList.Create;
  try
    GetVersionInfo(Application.ExeName, FVersionList);
  except
    on MFException do begin
      FHasInfo := False;
      {#todo1 Do I have to free FVersionList here?}
    end;
  end;

  FProduct := ReadProduct;
  FVersion := ReadVersion;
  FCopyright := ReadCopyright;
  FHomepage := ReadHomepage;
  FEmail := ReadEmail;
  FInfo := ReadInfo;

  FStyle := dsModern;
end;

destructor TMFAboutDialog.Destroy;
begin
  FVersionList.Free;
  inherited Destroy;
end;

procedure TMFAboutDialog.Execute;
begin
  AboutDlg := TMFAboutDlg.Create(Application);
  try
    AboutDlg.Title := FTitle;
    AboutDlg.Product := FProduct;
    AboutDlg.Version := FVersion;
    AboutDlg.Copyright := FCopyright;
    AboutDlg.Homepage := FHomepage;
    AboutDlg.Email := FEmail;
    AboutDlg.Info := FInfo;
    AboutDlg.Style := FStyle;

    AboutDlg.Position := poScreenCenter;

    AboutDlg.ShowModal;
  finally
    AboutDlg.Free;
  end;
end;

function TMFAboutDialog.ReadProduct: String;
begin
  if FHasInfo and (FVersionList[Integer(fiProductName)] <> '') then
    Result := FVersionList[Integer(fiProductName)]
  else
    Result := Application.Title;
end;

function TMFAboutDialog.ReadVersion: String;
begin
  if FHasInfo then
    Result := FVersionList[Integer(fiProductVersion)]
  else
    Result := '';
end;

function TMFAboutDialog.ReadCopyright: String;
begin
  if FHasInfo then
    Result := FVersionList[Integer(fiLegalCopyright)]
  else
    Result := '';
end;

function TMFAboutDialog.ReadHomepage: String;
begin
  if FHasInfo then
    Result := FVersionList[Integer(fiHomepage)]
  else
    Result := '';
end;

function TMFAboutDialog.ReadEmail: String;
begin
  if FHasInfo then
    Result := FVersionList[Integer(fiMail)]
  else
    Result := '';
end;

function TMFAboutDialog.ReadInfo: String;
begin
  if FHasInfo then
    Result := FVersionList[Integer(fiComments)]
  else
    Result := '';
end;

{ - Queries ------------------------------------------------------------------ }
function MFInputQuery(const Caption, Prompt: String; var Text: String): Boolean;
var InputQuery: TMFInputQuery;
begin
  InputQuery := TMFInputQuery.Create(nil);
  try
    InputQuery.Caption := Caption;
    InputQuery.laPrompt.Caption := Prompt;
    InputQuery.eText.Text := Text;

    if InputQuery.ShowModal = mrOK then begin
      Text := InputQuery.eText.Text;
      Result := True;
    end
    else begin
      Result := False;
    end;
  finally
    InputQuery.Free;
  end;
end;

function MFSelectQuery(const Caption, Prompt: String; const List: TStrings;
  const DefIndex: Integer; var Index: Integer): Boolean;
var SelQuery: TSelectQuery;
begin
  SelQuery := TSelectQuery.Create(nil);
  try
    SelQuery.Caption := Caption;
    SelQuery.laPrompt.Caption := Prompt;
    SelQuery.cbList.Items.AddStrings(List);
    if (DefIndex >= 0) and (DefIndex <= List.Count) then
      SelQuery.cbList.ItemIndex := DefIndex
    else
      SelQuery.cbList.ItemIndex := 0;
    SelQuery.cbList.OnChange(nil); // should work without, thought

    if SelQuery.ShowModal = mrOK then begin
      Index := SelQuery.cbList.ItemIndex;
      Result := True;
    end
    else begin
      Index := DefIndex;
      Result := False;
    end;
  finally
    SelQuery.Free;
  end;
end;

end.
