{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

Initial Autor   : Marcus Fuchs

Last Modified at: 12. January 2003
Last Modified by: Marcus Fuchs

You may retrieve the latest version of this file at the MFLib home page,
located at http://www.mfhome.net/mflib/
-------------------------------------------------------------------------------}

unit MFEdits;

interface

// remove the period to enable Debug settings (see MFLib.inc)
{.$DEFINE DEBUG}

{$I MFLib.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,     { VCL   }
  Dialogs, StdCtrls;                                                   { VCL   }

type
(*
  TZahlenbereich = (zbRationale {+/-0,1;+/-1},
                    zbGanze {+/-1},
                    zbNateurliche {+1},
                    zbNegNatuerliche {-1},
                    zbGebrochene {+0,1;+1},
                    zbNegGebrochene {-0,1;-1});
*)
  TNumbers = (nRational,     {0; 0 +/- n.n} {all numbers}
              nPosFraction,  {0 + n.n}      {nPosDecimalFraction}
              nNegFraction,  {0 - n.n}      {nNegDecimalFraction}
              nInteger,      {0 +/- n}      {not delphi's integer}
              nPosInteger,   {0 + n}        {nNatural}
              nNegInteger);  {0 - n}
  { Overview: }
  { Description: }

  TMFEdit = class(TEdit)
  { Overview: }
  { Description: All Edits of MFLib should be childs of TMFEdit.

    Note: For now TMFEdit is exactly TEdit, but in futur it could be a child of
    a new created TMFCustomEdit or TMFWinControl. }
  private
  protected
  public
  published
  end;

  TMFNumEdit = class(TMFEdit)
  { Overview: Edit that only allows numbers (not digits) to be entered. }
  { Description: Use this control if you want the user to enter a number.
    With the property Numbers you can specify the type of numbers you accept. } 
  { See also: TMFNumEdit.Numbers }
  private
    FNumbers: TNumbers;
    FAlignment: TAlignment;
    FBeep: Boolean;
    procedure SetAlignment(Value: TAlignment);
    function GetText: String;
    procedure SetText(const Value: String);
    function GetEditText: string;
    procedure SetEditText(const Value: String);
    function Validate(const Value: String): Boolean;
    procedure WMPaste(var Msg: TMessage); message WM_PASTE;
    procedure WMCut(var Msg: TMessage); message WM_Cut;
    procedure GetSel(var SelStart: Integer; var SelStop: Integer);
    procedure SetSel(SelStart: Integer; SelStop: Integer);
    property EditText: string read GetEditText write SetEditText;
    procedure CMExit(var Msg: TCMExit); message CM_EXIT;
    procedure SetBeep(Value: Boolean);
  protected
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure KeyPress(var Key: Char); override;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment
      default taRightJustify;
    property Beep: boolean read FBeep write SetBeep default True;
    property Text: string read GetText write SetText;
    property Numbers: TNumbers read FNumbers
      write FNumbers default nRational;
  end;


implementation

{$R ..\Resources\MFEdits.dcr}

Uses
  Clipbrd;

constructor TMFNumEdit.Create(AOwner: TComponent);
{ Overview: Die Methode Create erstellt eine Instanz von TMFNumEdit. }
{ Description: Rufen Sie Create auf, wenn Sie eine Instanz von TMFMNumEdit
  erstellen und initialisieren wollen.

  Nach dem Aufruf des geerbten Konstruktors initialisiert TMFNumEdit die
  nachstehenden Eigenschaften wie folgt:

   Numbers mit nRational.
   Alignment mit taRightJustify.
   Beep mit True.

  Abgeleitete Objekte, die die Methode Create berschreiben, mssen vor der
  Ausfhrung zustzlicher Initialisierungen die geerbte Methode Create aufrufen. }
begin
  inherited;
  FNumbers := nRational;
  FAlignment := taRightJustify;
  FBeep := True;
end;

procedure TMFNumEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  case Alignment of
    taLeftJustify : Params.Style := Params.Style or ES_LEFT and not ES_MULTILINE;
    taRightJustify: Params.Style := Params.Style or ES_RIGHT and not ES_MULTILINE;
    taCenter      : Params.Style := Params.Style or ES_CENTER and not ES_MULTILINE;
  end;
end;

procedure TMFNumEdit.SetAlignment(Value: TAlignment);
begin
  if FAlignment <> Value then begin
    FAlignment := Value;
    RecreateWnd;
  end;
end;

function TMFNumEdit.GetText: String;
begin
  Result := inherited Text;
end;

procedure TMFNumEdit.SetText(const Value: String);
begin
  if GetEditText <> Value then begin
    if Validate(Value) then // ((csDesigning in ComponentState) and not (csLoading in ComponentState))
      EditText := Value
    else
      If (csDesigning in ComponentState) Then
        MessageBeep(0)
  end;
end;

function TMFNumEdit.GetEditText: string;
begin
  Result := inherited Text;
end;

procedure TMFNumEdit.SetEditText(const Value: String);
begin
  inherited Text := Value;
end;

function TMFNumEdit.Validate(const Value: String): Boolean;
var i: Integer;
    Key: Char;
    IsAllreadyDecimal: Boolean;
begin
  Result := True;
  IsAllreadyDecimal := False;
  for i := 1 to Length(Value) do begin
    Key := Value[i];
    if (Key in ['0'..'9']) or
       ((FNumbers in [nRational, nInteger, nNegInteger,
         nNegFraction]) and (Key = '-')) or
       ((FNumbers in [nRational, nPosFraction, nNegFraction]) and
        (Key = DecimalSeparator)) then begin
      if Key = DecimalSeparator then
        if (IsAllreadyDecimal) or (Pos(Key, Value) = 1) or
           ((Pos(Key, Value) = 2) and (Pos('-', Value) = 1)) then begin
          Result := False;
          Exit;
        end
        else
          IsAllreadyDecimal := True
      else
        If (FNumbers in [nNegInteger, nNegFraction]) and (i = 1) and
            not(Key = '-') then begin
          Result := False;
          Exit;
        end
        else
          if (Key = '-') and (i > 1) then begin
            Result := False;
            Exit;
          end;
    end
    else begin
      Result := False;
      Exit;
    end;
  end;
end;

procedure TMFNumEdit.KeyPress(var Key: Char);
begin
  if not( (Key in ['0'..'9']) or
          ((FNumbers in [nRational, nInteger, nNegInteger,
            nNegFraction]) and (Key = '-')) or
          ((FNumbers in [nRational, nPosFraction, nNegFraction]) and
           (Key = DecimalSeparator))) then
    Key := #0 { 'Invalid key: ' + Key }
  else
    if ((Key = DecimalSeparator) or (Key = '-')) and (Pos(Key, Text) > 0) and
       ((SelStart+1 > Pos(Key, Text)) or ((SelStart+1 <= Pos(Key, Text)) and
       (SelStart+SelLength < Pos(Key,Text)))) then
      Key:=#0 { 'Invalid Key: twice ' + Key }
    else
      if (Key = DecimalSeparator) and (SelStart = 0) then
        Key := #0 { 'Not allowed at beginning of number: ' + Key }
      else
        if (Key = DecimalSeparator) and (Pos('-', Text) = SelStart) then
          Key := #0 { 'Not allowed directly after "-": ' + Key }
        else
          if (Key = '-') and (Length(Text) > 0) and
             (Text[SelStart+SelLength+1] = DecimalSeparator) then
            Key := #0 { 'Not allowed directly befor ' + DecimalSeparator + ': ' + Key }
          else
            if (Key = '-') and (SelStart <> 0) then
              Key := #0 { 'Only allowed at beginning of number: ' + Key }
            else
              if (FNumbers in [nNegInteger, nNegFraction])  and
                 (SelStart = 0) and not(Key = '-') then
                Key := #0; { '"-" must be the first Key if only negativ numbers' }
end;

procedure TMFNumEdit.WMPaste(var Msg: TMessage);
var Value, Str, OldValue: String;
    SelStart, SelStop : Integer;
begin
  if ReadOnly then
    inherited
  else begin
    Clipboard.Open;
    Value := Clipboard.AsText;
    Clipboard.Close;

    OldValue:=EditText;
    GetSel(SelStart, SelStop);
    Str := EditText;
    System.Delete(Str, Selstart+1, SelStop-SelStart);
    EditText := Str;
    System.Insert(Value, str, SelStart+1);
    EditText := Str;
    SetSel(SelStart, SelStart+Length(Value));
    if not Validate(Str) then begin
      EditText := OldValue;
      if FBeep then
        MessageBeep(0);
      SetSel(SelStart, SelStop);
    end;
  end;
end;

procedure TMFNumEdit.WMCut(var Msg: TMessage);
var SelStart, SelStop: Integer;
    Str, OldValue: String;
begin
  CopyToClipboard;
  if ReadOnly then Exit;

  GetSel(SelStart, SelStop);
  if (SelStop - SelStart) < 1 then Exit;
  OldValue := EditText;
  Str := EditText;
  Delete(Str, SelStart+1, SelStop-SelStart);
  EditText := Str;
  SetSel(SelStart, SelStart);
  if not Validate(Str) then begin
    EditText := OldValue;
    if FBeep then
      MessageBeep(0);
    SetSel(SelStart, SelStop);
  end;
end;

procedure TMFNumEdit.GetSel(var SelStart: Integer; var SelStop: Integer);
begin
  SendMessage(Handle, EM_GETSEL, Integer(@SelStart), Integer(@SelStop));
end;

procedure TMFNumEdit.SetSel(SelStart: Integer; SelStop: Integer);
begin
  SendMessage(Handle, EM_SETSEL, SelStart, SelStop);
end;

procedure TMFNumEdit.CMExit(var Msg: TCMExit);
begin
  if Modified then begin
    if not Validate(EditText) then begin
      if not (csDesigning in ComponentState) then
        SetFocus;
      if FBeep then
        MessageBeep(0);
    end;
  end;
  inherited;
end;

procedure TMFNumEdit.KeyDown(var Key: Word; Shift: TShiftState);
var Str: String;
    Pos, Len: Integer;
begin
 inherited KeyDown(Key, Shift);
 Str := EditText;
 if ((Key = VK_DELETE) and ([ssShift, ssCtrl] * Shift = [])) then begin
   Pos := SelStart;
   Len := SelLength;
   if Len = 0 then
     Len := 1;
   Delete(Str, SelStart+1, Len);
   if Validate(Str) then
     EditText := Str;
   SetSel(Pos, Pos);
   Key := 0;
 end
 else
   if (Key = VK_BACK) then begin
     Len := SelLength;
     if Len = 0 then begin
       Pos := Selstart-1; // verursacht Schnheitsfehler wenn invalid
       Delete(Str, SelStart, Len+1);
     end
     else begin
       Pos := Selstart;
       Delete(Str, SelStart+1, Len);
     end;
     if Validate(Str) then
       EditText := Str;
     SetSel(Pos, Pos);
     Key := 0;
   end;
end;

procedure TMFNumEdit.SetBeep(Value: Boolean);
begin
  if FBeep <> Value then
    FBeep := Value
end;

end.
