{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

Initial Autor   : Marcus Fuchs

Last Modified at: 05. February 2003
Last Modified by: Marcus Fuchs

You may retrieve the latest version of this file at the MFLib home page,
located at http://www.mfhome.net/mflib/
-------------------------------------------------------------------------------}

unit MFLabels;

interface { ------------------------------------------------------ interface - }

// remove the period to enable Debug settings (see MFLib.inc)
{.$DEFINE DEBUG}

{$I MFLib.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, ExtCtrls,                                           { VCL   }
  MFTypes;                                                             { MFLib }

type
  TLabelBevel = TPanelBevel;
  TEllipsesStyle = (esNone, esEnd, esPath);

type
  TMFCustomLabel = class(TCustomLabel)
  private
    FAutoSize: Boolean;
    FWordWrap: Boolean;
    FBorder: TBorderStyle;
    FBevel: TLabelBevel;
    FEscapement: Integer; { = Angle }
    FEllipsesStyle: TEllipsesStyle;
    FMouseInPos: Boolean;
    FShadowIndent: Integer;
    FTextColors: TMFTextColors;
    FTextStyle: TMFTextStyle;

    FOnMouseEnter: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;

    mUp: Boolean;

    procedure AdjustBounds;

    procedure CMMouseEnter(var Msg:TMessage); message CM_MouseEnter;
    procedure CMMouseLeave(var Msg:TMessage); message CM_MouseLeave;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;

    procedure SetBorder(Value: TBorderStyle);
    procedure SetBevel(Value: TLabelBevel);
    procedure SetEscapement(aVal: Integer);
    procedure SetEllipsesStyle(Value: TEllipsesStyle);
    procedure SetShadowIndent(Value: Integer);
    procedure SetTextColors(Value: TMFTextColors);
    procedure SetTextStyle(aVal: TMFTextStyle);
    procedure SetWordWrap(Value: Boolean);
    function GetTransparent: Boolean;
    procedure SetTransparent(Value: Boolean);
    procedure CalcTextPos(var aRect: TRect; aAngle: Integer; aTxt: String);
    procedure DrawAngleText(aCanvas: TCanvas; aRect: TRect; aAngle:
      Integer; aTxt: String);
    procedure OnTextColorChanged(Sender: TObject);
  protected
    procedure Loaded; override;
    procedure DoDrawText(var Rect: TRect; Flags: Word); virtual;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Paint; override;
    procedure SetAutoSize(Value: Boolean); override;

    property AutoSize: Boolean read FAutoSize write SetAutoSize default True;
    property Border: TBorderStyle read FBorder write SetBorder default bsNone;
    property Bevel: TLabelBevel read FBevel write SetBevel default bvNone;
    property Escapement: Integer read fEscapement write SetEscapement; {#todo2 Escapement - check the use}
    property EllipsesStyle: TEllipsesStyle read FEllipsesStyle
      write SetEllipsesStyle default esNone;
    property ShadowIndent: Integer read FShadowIndent write SetShadowIndent
      default 1;
    property TextColors: TMFTextColors read FTextColors write SetTextColors;
    property TextStyle: TMFTextStyle read fTextStyle  write SetTextStyle
      default tsRaised;
    property Transparent: Boolean read GetTransparent write SetTransparent
      default True;
    property WordWrap: Boolean read FWordWrap write SetWordWrap default False;

    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  public
    constructor Create(AOwner: TComponent); override;
  published
  end;

  TMFLabel = class(TMFCustomLabel)
  private
  protected
  public
    property MouseInPos: Boolean read FMouseInPos;
  published
    property Align;
    property Alignment;
    property AutoSize;
    property Bevel;
    property Border;
    property Caption;
    property Color;
    property DragCursor;
    property DragMode;
    Property EllipsesStyle;
    property Enabled;
{#todo2 Escapement - publish after testing
    property Escapement;     }
    property FocusControl;
    property Font;
    Property Layout;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShadowIndent;
    property ShowAccelChar;
    property ShowHint;
    property TextColors;
    property TextStyle;
    property Transparent;
    property Visible;
    property WordWrap;

    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

  TMFUrlLabel = class(TMFLabel)
  private
    FUrl: String;
    FNewWindow: Boolean;
    FPopupMenu: TPopupMenu;
    procedure CopyLink(Sender: TObject);
    procedure SurfToUrl(Sender: TObject);
    procedure CMMouseEnter(var Msg:TMessage); message CM_MouseEnter;
  protected
    procedure Click; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure SetUrl(Value: String); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property NewWindow: Boolean read FNewWindow write FNewWindow default True;
    property Url: String read FUrl write SetUrl;
    property OnClick;
  end;


implementation { -------------------------------------------- implementation - }

{$R ..\Resources\MFLabels.dcr}

Uses
  ShellAPI, Clipbrd,                                                   { VCL   }
  MFRoutines, MFStrRes;                                                { MFLib }

{ - TMFCustomLabel ----------------------------------------------------------- }

constructor TMFCustomLabel.Create(aOwner:TComponent);
begin
  inherited Create(aOwner);
  FAutoSize := True;
  FBorder := bsNone;
  FEscapement:= 0;
  FEllipsesStyle := esNone;
  FShadowIndent := 1;
  FTextStyle := tsRaised;
  FWordWrap := False;

  Transparent := True;

  FTextColors := TMFTextColors.Create;
  FTextColors.OnChanged := OnTextColorChanged;
  Font.Color := FTextColors.Colors.Color;

  mUp := True;
end;

procedure TMFCustomLabel.OnTextColorChanged(Sender: TObject);
begin
  Font.Color := FTextColors.Colors.Color;
  Invalidate;
end;

procedure TMFCustomLabel.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
  AdjustBounds;
end;

procedure TMFCustomLabel.CMFontChanged(var Message: TMessage);
begin
  inherited;
  FTextColors.Colors.Color := Font.Color;
  AdjustBounds;
  Invalidate;
end;

procedure TMFCustomLabel.SetBorder(Value: TBorderStyle);
begin
  if (FBorder <> Value) then begin
    FBorder := Value;
    Invalidate;
  end;
end;

procedure TMFCustomLabel.SetBevel(Value: TLabelBevel);
begin
  if (FBevel <> Value) then begin
    FBevel := Value;
    Invalidate;
  end;
end;

procedure TMFCustomLabel.SetTextColors(Value: TMFTextColors);
begin
//  inherited; // not needed
  FTextColors.Assign(Value);
  Font.Color := Value.Colors.Color;
  Invalidate;
end;

procedure TMFCustomLabel.SetEscapement(aVal:Integer);
begin
  if fEscapement <> aVal then begin
     if aVal < 0 then begin
       while aVal < -360 do aVal := aVal + 360;
       aVal := 360 + aVal;
     end;
     while aVal > 360 do aVal := aVal - 360;
     fEscapement := aVal;
     Invalidate;
  end;
end;

procedure TMFCustomLabel.SetAutoSize(Value: Boolean);
begin
//  inherited; // not needed
  if FAutoSize <> Value then begin
    FAutoSize := Value;
    AdjustBounds;
  end;
end;

procedure TMFCustomLabel.SetWordWrap(Value: Boolean);
begin
  if FWordWrap <> Value then begin
    FWordWrap := Value;
    AdjustBounds;
    Invalidate;
  end;
end;

procedure TMFCustomLabel.SetTextStyle(aVal:TMFTextStyle);
begin
  if fTextStyle <> aVal then begin
    fTextStyle := aVal;
    Invalidate;
  end;
end;

procedure TMFCustomLabel.Paint;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  WordWraps : array[Boolean] of Word = (0, DT_WORDBREAK);
var
  CtrlRect, RectBorder: TRect;
begin
  with Canvas do begin
    if not Transparent then begin
      Brush.Color := Self.Color;
      Brush.Style := bsSolid;
      FillRect(ClientRect);
    end;
    Brush.Style := bsClear;
    CtrlRect := ClientRect;

    if FBevel <> bvNone then begin
      RectBorder := Rect(CtrlRect.left, CtrlRect.top, CtrlRect.right, CtrlRect.bottom);
      case FBevel of
        bvRaised : Frame3D(Canvas, RectBorder, clBtnHighlight, clBtnShadow, 1);
        bvLowered: Frame3D(Canvas, RectBorder, clBtnShadow, clBtnHighlight, 1);
      end;
    end;

    if FBorder = bsSingle then begin
      if FBevel = bvNone then
        RectBorder := Rect(CtrlRect.left, CtrlRect.top, CtrlRect.right, CtrlRect.bottom);
      Frame3D(Canvas, RectBorder, clWindowFrame, clWindowFrame, 1);
    end;

    DoDrawText(CtrlRect, DT_EXPANDTABS or WordWraps[FWordWrap] or Alignments[Alignment]);

    {#todo3 if Align <> alNone then Autosize = False}
    if FAutoSize then begin
      Autosize := False;
      Autosize := True;
    end;
  end;
end;

procedure TMFCustomLabel.Loaded;
begin
  inherited; // {#todo3 not needed}
  AdjustBounds;
end;

procedure TMFCustomLabel.AdjustBounds;
const
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);
var
  DC: HDC;
  X, diff: Integer;
  cRect: TRect;
begin
  if not (csReading in ComponentState) and FAutoSize then
  begin
    cRect := ClientRect;
    diff := 0;
    if Border <> bsNone then
      diff := diff + 1;
    if Bevel <> bvNone then
      diff := diff + 1;
{ The text should not start at P(0,0) but on P(dff,diff) }
//    OffsetRect(cRect, diff, diff);
    DC := GetDC(0);
    Canvas.Handle := DC;
    DoDrawText(cRect, (DT_EXPANDTABS or DT_CALCRECT) or WordWraps[FWordWrap]);
    Canvas.Handle := 0;
    ReleaseDC(0, DC);
    X := Left;
    if {F}Alignment = taRightJustify then Inc(X, Width - cRect.Right);
    SetBounds(X, Top, cRect.Right + diff, cRect.Bottom + diff);
  end;
end;

procedure TMFCustomLabel.CalcTextPos(var aRect: TRect; aAngle: Integer; aTxt: String);
var DC      : HDC;
    hSavFont: HFont;
    Size    : TSize;
    x,y     : Integer;
begin
  DC := GetDC(0);
  hSavFont := SelectObject(DC, Font.Handle);
  GetTextExtentPoint32(DC, pChar(aTxt), Length(aTxt), Size);
  SelectObject(DC, hSavFont);
  ReleaseDC(0, DC);

  x := 0; // so that the compiler doesn't give a warning on uninitialised vars
  y := 0;

  if          aAngle <= 90  then begin             { 1.Quadrant }
     x := 0;
     y := Trunc(Size.cx * sin(aAngle*Pi/180));
  end else if aAngle <= 180 then begin             { 2.Quadrant }
     x := Trunc(Size.cx * -cos(aAngle*Pi/180));
     y := Trunc(Size.cx *  sin(aAngle*Pi/180) + Size.cy * cos((180-aAngle)*Pi/180));
  end else if aAngle <= 270 then begin             { 3.Quadrant }
     x := Trunc(Size.cx * -cos(aAngle*Pi/180) + Size.cy * sin((aAngle-180)*Pi/180));
     y := Trunc(Size.cy * sin((270-aAngle)*Pi/180));
  end else if aAngle <= 360 then begin             { 4.Quadrant }
     x := Trunc(Size.cy * sin((360-aAngle)*Pi/180));
     y := 0;
  end;

  aRect.Top := aRect.Top  + y;
  aRect.Left:= aRect.Left + x;

  x := Abs(Trunc(Size.cx * cos(aAngle*Pi/180))) + Abs(Trunc(Size.cy * sin(aAngle*Pi/180)));
  y := Abs(Trunc(Size.cx * sin(aAngle*Pi/180))) + Abs(Trunc(Size.cy * cos(aAngle*Pi/180)));

  if FAutosize then begin
     Width  := x;
     Height := y;
  end else if Alignment = taCenter then begin
     aRect.Left:= aRect.Left + ((Width-x) div 2);
  end else if Alignment = taRightJustify then begin
     aRect.Left:= aRect.Left + Width - x;
  end;
end;

{#todo2 DrawAngleText does not work as it should. Compare with original code.}
procedure TMFCustomLabel.DrawAngleText(aCanvas: TCanvas; aRect: tRect; aAngle: Integer; aTxt: String);
var LFont             : TLogFont;
    hOldFont, hNewFont: HFont;
begin
  CalcTextPos(aRect, aAngle, aTxt);

  GetObject(aCanvas.Font.Handle, SizeOf(LFont), Addr(LFont));
  LFont.lfEscapement := aAngle*10;
  hNewFont := CreateFontIndirect(LFont);
  hOldFont := SelectObject(aCanvas.Handle, hNewFont);

  aCanvas.TextOut(aRect.Left, aRect.Top, aTxt);

  hNewFont := SelectObject(aCanvas.Handle, hOldFont);
  DeleteObject(hNewFont);
end;

procedure TMFCustomLabel.DoDrawText(var Rect: TRect; Flags: Word);
var Text: String;
    TmpRect: TRect;
    NormalColor, UpperColor, LowerColor: TColor;
begin
  Text := Caption;

  if (Flags and DT_CALCRECT <> 0) and ((Text = '') or ShowAccelChar and
    (Text[1] = '&') and (Text[2] = #0)) then Text := Text + ' ';

  if not ShowAccelChar then Flags := Flags or DT_NOPREFIX;
  If FEllipsesStyle = esEnd Then
    Flags := Flags Or DT_END_ELLIPSIS
  Else
    If FEllipsesStyle = esPath Then
      Flags := Flags Or DT_PATH_ELLIPSIS;
  If FEscapement = 0 Then
    Case Layout of
      tlTop   : Flags := Flags or DT_TOP;
      tlCenter: Flags := Flags or DT_VCENTER or DT_SINGLELINE;
      tlBottom: Flags := Flags or DT_BOTTOM or DT_SINGLELINE;
    End;

  Canvas.Font := Font;

  If FEscapement in [46..134] Then  ;
  {special manipulation with shadow position is unsupported- J.B.}

  if FMouseInPos then begin
    if mUp then begin
      NormalColor := FTextColors.Colors.HoverColor;
      UpperColor := FTextColors.LightColors.HoverColor;
      LowerColor := FTextColors.ShadowColors.HoverColor;
    end
    else begin
      NormalColor := FTextColors.Colors.DownColor;
      UpperColor := FTextColors.LightColors.DownColor;
      LowerColor := FTextColors.ShadowColors.DownColor;
    end;
  end
  else begin
    NormalColor := FTextColors.Colors.Color;
    UpperColor := FTextColors.LightColors.Color;
    LowerColor := FTextColors.ShadowColors.Color;
  end;

  if FTextStyle <> tsNone then begin
    { DoLoweredColors }
    if FTextStyle in [tsLowered, tsHeavyLowered, tsHeavyRaised, tsShadow] then begin
      TmpRect := Rect;
      If FTextStyle <> tsShadow Then
        OffsetRect(TmpRect, 1, 1)
      Else
        OffsetRect(TmpRect, FShadowIndent, FShadowIndent);
      Case FTextStyle of
        tsLowered     : Canvas.Font.Color := UpperColor;
        tsHeavyLowered: Canvas.Font.Color := UpperColor;
        tsHeavyRaised : Canvas.Font.Color := LowerColor;
        tsShadow      : Canvas.Font.Color := LowerColor;
      End;
      if fEscapement <> 0 then
        DrawAngleText(Canvas, TmpRect, fEscapement, Text)
      else
        DrawText(Canvas.Handle, pChar(Text), Length(Text), TmpRect, Flags);
    end;

    { DoRaisedColrs }
    if FTextStyle in [tsHeavyLowered, tsRaised, tsHeavyRaised] then begin
      TmpRect := Rect;
      OffsetRect(TmpRect, -1, -1);
      Case FTextStyle of
        tsHeavyLowered: Canvas.Font.Color := LowerColor;
        tsRaised      : Canvas.Font.Color := UpperColor;
        tsHeavyRaised : Canvas.Font.Color := UpperColor;
      End;
      if fEscapement <> 0 then
        DrawAngleText(Canvas, TmpRect, fEscapement, Text)
      else
        DrawText(Canvas.Handle, pChar(Text), Length(Text), TmpRect, Flags);
    end
  end;

  if not Enabled then
    Canvas.Font.Color := FTextColors.DisabledColor
  else
    Canvas.Font.Color := NormalColor;

  if fEscapement <> 0 then
    DrawAngleText(Canvas, Rect, FEscapement, Text)
  else
    DrawText(Canvas.Handle, pChar(Text), Length(Text), Rect, Flags);
end;

Procedure TMFCustomLabel.SetEllipsesStyle(Value: TEllipsesStyle);
Begin
  If FEllipsesStyle <> Value Then Begin
    if Value <> esNone then
      AutoSize := False;
    FEllipsesStyle := Value;
    Invalidate;
  End;
End;

Procedure TMFCustomLabel.SetShadowIndent(Value: Integer);
Begin
  If Value <> FShadowIndent Then Begin
    FShadowIndent := Value;
    Invalidate;
  End;
End;

function TMFCustomLabel.GetTransparent: Boolean;
begin
  Result := not (csOpaque in ControlStyle);
end;

procedure TMFCustomLabel.SetTransparent(Value: Boolean);
begin
  if Transparent <> Value then begin
    if Value then
      ControlStyle := ControlStyle - [csOpaque]
    else
      ControlStyle := ControlStyle + [csOpaque];
    Invalidate;
  end;
end;

procedure TMFCustomLabel.CMMouseEnter(var Msg: TMessage);
begin
  FMouseInPos := True;
  if Assigned(FOnMouseEnter) then
    FOnMouseEnter(self)
  else
    Invalidate;
end;

procedure TMFCustomLabel.CMMouseLeave(var Msg: TMessage);
begin
  FMouseInPos := False;
  if Assigned(FOnMouseLeave) then
    FOnMouseLeave(self)
  else
    Invalidate;
end;

procedure TMFCustomLabel.MouseDown(Button: TMouseButton; Shift: TShiftState;
   X, Y: Integer);
begin
  inherited;
  if not mUp then exit; // don't do more than once
  if Button = mbLeft then mUp := False;
  Invalidate;
end;

procedure TMFCustomLabel.MouseUp(Button: TMouseButton; Shift: TShiftState;
   X, Y: Integer);
begin
  inherited;
  if mUp then exit; // don't do more than once
  mUp := True;
  Invalidate;
end;

{ - TMFLabel ----------------------------------------------------------------- }


{ - TMFUrlLabel -------------------------------------------------------------- }

constructor TMFUrlLabel.Create(AOwner: TComponent);
var
  FMenuItem1, FMenuItem2: TMenuItem;
begin
  inherited Create(AOwner);
  FTextColors.Colors.Color := clBlue;
  FTextColors.Colors.HoverColor := clRed;

  FNewWindow := True;
  OnClick := SurfToUrl;

  FMenuItem1 := TMenuItem.Create(self);
  FMenuItem1.Caption := srURLLabelOpen;
  FMenuItem1.OnClick := SurfToUrl;
  FMenuItem2 := TMenuItem.Create(self);
  FMenuItem2.Caption := srURLLabelCopyLink;
  FMenuItem2.OnClick := CopyLink;
  FPopupMenu := TPopupMenu.Create(self);
  FPopupMenu.Items.Add(FMenuItem1);
  FPopupMenu.Items.Add(FMenuItem2);
end;

destructor TMFUrlLabel.Destroy;
begin
  FPopupMenu.Free;
  inherited;
end;

procedure TMFUrlLabel.Click;
begin
  inherited;
end;

procedure TMFUrlLabel.MouseDown(Button: TMouseButton; Shift: TShiftState;
   X, Y: Integer);
begin
  inherited;
  if (Button = mbRight) and (PopupMenu = nil) then
    PopupMenu := FPopupMenu;
end;

procedure TMFUrlLabel.SetUrl(Value: String);
type TProtocol = (urlhttp, urlftp, urlfile, urlmailto, urlnews, urlnntp,
   urltelnet, urlgopher, urlwais, urlprospero);
const Protocol: Array[TProtocol] of String = ('http://', 'ftp://', 'file://',
   'mailto:', 'news:', 'nntp://', 'telnet://', 'gopher://', 'wais://',
   'prospero://');
var i: TProtocol;
begin
  if Pos('www.', Value) = 1 then
    Value := Protocol[urlhttp] + Value;
  for i := high(Protocol) downto low(Protocol) do
    if Pos(Protocol[i], Value) > 0 then begin
      FUrl := Value;
      Exit;
    end;
  if FUrl <> Value then
    raise MFException.CreateFmt(srENoValidLink, [Value]);
end;

procedure TMFUrlLabel.CMMouseEnter(var Msg: TMessage);
begin
  inherited;
  Cursor := crHandPoint;
end;

procedure TMFUrlLabel.CopyLink(Sender: TObject);
begin
  Clipboard.SetTextBuf(PChar(FUrl)); {#todo1 Memoryleak?}
end;

procedure TMFUrlLabel.SurfToUrl(Sender: TObject);
begin
  if FNewWindow then
    ShellExecute(Application.Handle, 'open',
       PChar(GetExeForProtocol(PChar(FUrl))), PChar(FUrl), nil, SW_SHOWNORMAL)
  else
    ShellExecute(Application.Handle, 'open', PChar(FUrl), nil, nil, SW_SHOWNORMAL);
end;

end.
