{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

Initial Autor   : Marcus Fuchs

Last Modified at: 14. January 2003
Last Modified by: Marcus Fuchs

You may retrieve the latest version of this file at the MFLib home page,
located at http://www.mfhome.net/mflib/
-------------------------------------------------------------------------------}

unit MFMessages;

interface { ------------------------------------------------------ interface - }

// remove the period to enable Debug settings (see MFLib.inc)
{.$DEFINE DEBUG}

{$I MFLib.inc}

Uses
  Controls;                                                            {  VCL  }

type
  TMsgDlgExType = (mtWarning, mtError, mtInformation, mtConfirmation, mtCustom);

  TMsgDlgExBtn = (mbYes, mbNo, mbOK, mbCancel, mbAbort, mbRetry, mbIgnore,
     mbAll, mbNoToAll, mbYesToAll, mbHelp, mbUpdate, mbRegister, mbRemind,
     mbMore);

  TMsgDlgExButtons = set of TMsgDlgExBtn;

const
  mrUpdate = mrYesToAll + 1;
  mrRegister = mrUpdate + 1;
  mrRemind = mrRegister + 1;
  mrMore = mrRemind + 1;

const
  mbUpdateMoreCancel = [mbUpdate, mbMore, mbCancel];
  mbRegisterRemind = [mbRegister, mbRemind];

function MessageDlgEx(const aCaption: String; const Msg: string;
  DlgType: TMsgDlgExType; Buttons: TMsgDlgExButtons;
  DefaultButton, CancelButton: TMsgDlgExBtn; HelpCtx: Longint): Integer;
{ Description: MessageDlgEx is an advanced MessageDlg with the following
  improvements compared to the standard Delphi one:
  - uses advanced Buttontype
  - you can set a custom caption (if it is an empty string, standard captions
    will be used)
  - you can set the defaultButton as well as the cancelButton
}


implementation { -------------------------------------------- implementation - }

uses
  Windows, Classes, Forms, Graphics, StdCtrls, ExtCtrls,               {  VCL  }
  MFUtils, MFStrRes;                                                   { MFLib }

////////////////////////////////////////////////////////////////////////////////

function Max(I, J: Integer): Integer;
begin
  if I > J then Result := I else Result := J;
end;

////////////////////////////////////////////////////////////////////////////////

function GetAveCharSize(Canvas: TCanvas): TPoint;
var
  I: Integer;
  Buffer: array[0..51] of Char;
begin
  for I := 0 to 25 do Buffer[I] := Chr(I + Ord('A'));
  for I := 0 to 25 do Buffer[I + 26] := Chr(I + Ord('a'));
  GetTextExtentPoint(Canvas.Handle, Buffer, 52, TSize(Result));
  Result.X := Result.X div 52;
end;

// -----------------------------------------------------------------------------
// TMessageForm = class(TForm)
// -----------------------------------------------------------------------------

type
  TMessageForm = class(TForm)
  private
    procedure HelpButtonClick(Sender: TObject);
  public
    constructor CreateNew(AOwner: TComponent);
  end;

constructor TMessageForm.CreateNew(AOwner: TComponent);
var
  NonClientMetrics: TNonClientMetrics;
begin
  inherited CreateNew(AOwner);
  NonClientMetrics.cbSize := sizeof(NonClientMetrics);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0, @NonClientMetrics, 0) then
    Font.Handle := CreateFontIndirect(NonClientMetrics.lfMessageFont);
end;

procedure TMessageForm.HelpButtonClick(Sender: TObject);
begin
  Application.HelpContext(HelpContext);
end;

////////////////////////////////////////////////////////////////////////////////

var
  Captions: array[TMsgDlgExType] of Integer = (srMsgDlgWarning, srMsgDlgError,
     srMsgDlgInformation, srMsgDlgConfirm, srMsgDlgCustom);

  IconIDs: array[TMsgDlgExType] of PChar = (IDI_EXCLAMATION, IDI_HAND,
     IDI_ASTERISK, IDI_QUESTION, nil);


  ExButtonNames: array[TMsgDlgExBtn] of string = ('Yes', 'No', 'OK', 'Cancel',
     'Abort', 'Retry', 'Ignore', 'All', 'NoToAll', 'YesToAll', 'Help', 'Update',
     'Register', 'Remind', 'More');

  ExButtonCaptions: array[TMsgDlgExBtn] of Integer = (srMsgDlgYes, srMsgDlgNo,
     srMsgDlgOK, srMsgDlgCancel, srMsgDlgAbort, srMsgDlgRetry, srMsgDlgIgnore,
     srMsgDlgAll, srMsgDlgNoToAll, srMsgDlgYesToAll, srMsgDlgHelp,
     srMsgDlgExUpdate, srMsgDlgExRegister, srMsgDlgExRemind, srMsgDlgExMore);

  ExModalResults: array[TMsgDlgExBtn] of Integer = (mrYes, mrNo, mrOk, mrCancel,
     mrAbort, mrRetry, mrIgnore, mrAll, mrNoToAll, mrYesToAll, 0, mrUpdate,
     mrRegister, mrRemind, mrMore);

////////////////////////////////////////////////////////////////////////////////

function CreateMessageDialogEx(const Msg: string; DlgType: TMsgDlgExType;
  Buttons: TMsgDlgExButtons; DefaultButton, CancelButton: TMsgDlgExBtn): TForm;
const
  mcHorzMargin = 8;
  mcVertMargin = 8;
  mcHorzSpacing = 10;
  mcVertSpacing = 10;
  mcButtonWidth = 65; // Delphi uses here 50
  mcButtonHeight = 14;
  mcButtonSpacing = 4;
var
  DialogUnits: TPoint;
  HorzMargin, VertMargin, HorzSpacing, VertSpacing, ButtonWidth,
  ButtonHeight, ButtonSpacing, ButtonCount, ButtonGroupWidth,
  IconTextWidth, IconTextHeight, X: Integer;
  B: TMsgDlgExBtn;
  IconID: PChar;
  TextRect: TRect;
begin
  Result := TMessageForm.CreateNew(Application);
  with Result do
  begin
    BorderStyle := bsDialog;
    Canvas.Font := Font;
    DialogUnits := GetAveCharSize(Canvas);
    HorzMargin := MulDiv(mcHorzMargin, DialogUnits.X, 4);
    VertMargin := MulDiv(mcVertMargin, DialogUnits.Y, 8);
    HorzSpacing := MulDiv(mcHorzSpacing, DialogUnits.X, 4);
    VertSpacing := MulDiv(mcVertSpacing, DialogUnits.Y, 8);
    ButtonWidth := MulDiv(mcButtonWidth, DialogUnits.X, 4);
    ButtonHeight := MulDiv(mcButtonHeight, DialogUnits.Y, 8);
    ButtonSpacing := MulDiv(mcButtonSpacing, DialogUnits.X, 4);
    SetRect(TextRect, 0, 0, Screen.Width div 2, 0);
    DrawText(Canvas.Handle, PChar(Msg), Length(Msg), TextRect,
      DT_EXPANDTABS or DT_CALCRECT or DT_WORDBREAK);
    IconID := IconIDs[DlgType];
    IconTextWidth := TextRect.Right;
    IconTextHeight := TextRect.Bottom;
    if IconID <> nil then begin
      Inc(IconTextWidth, 32 + HorzSpacing);
      if IconTextHeight < 32 then IconTextHeight := 32;
    end;
    ButtonCount := 0;
    for B := Low(TMsgDlgExBtn) to High(TMsgDlgExBtn) do
      if B in Buttons then Inc(ButtonCount);
    ButtonGroupWidth := 0;
    if ButtonCount <> 0 then
      ButtonGroupWidth := ButtonWidth * ButtonCount +
        ButtonSpacing * (ButtonCount - 1);
    ClientWidth := Max(IconTextWidth, ButtonGroupWidth) + HorzMargin * 2;
    ClientHeight := IconTextHeight + ButtonHeight + VertSpacing +
      VertMargin * 2;
    Left := (Screen.Width div 2) - (Width div 2);
    Top := (Screen.Height div 2) - (Height div 2);
    if DlgType <> mtCustom then
      Caption := ResStr(Captions[DlgType])
    else
      Caption := Application.Title;
    if IconID <> nil then
      with TImage.Create(Result) do begin
        Name := 'Image';
        Parent := Result;
        Picture.Icon.Handle := LoadIcon(0, IconID);
        SetBounds(HorzMargin, VertMargin, 32, 32);
      end;
    with TLabel.Create(Result) do begin
      Name := 'Message';
      Parent := Result;
      WordWrap := True;
      Caption := Msg;
      BoundsRect := TextRect;
      SetBounds(IconTextWidth - TextRect.Right + HorzMargin, VertMargin,
        TextRect.Right, TextRect.Bottom);
    end;

    X := (ClientWidth - ButtonGroupWidth) div 2;
    for B := Low(TMsgDlgExBtn) to High(TMsgDlgExBtn) do
      if B in Buttons then
        with TButton.Create(Result) do begin
          Name := ExButtonNames[B];
          Parent := Result;
          Caption := ResStr(ExButtonCaptions[B]);
          ModalResult := ExModalResults[B];
          if B = DefaultButton then
            Default := True
          else
            if B = CancelButton then
              Cancel := True;
          SetBounds(X, IconTextHeight + VertMargin + VertSpacing,
            ButtonWidth, ButtonHeight);
          Inc(X, ButtonWidth + ButtonSpacing);
          if B = mbHelp then
            OnClick := TMessageForm(Result).HelpButtonClick;
        end;
  end;
end;

////////////////////////////////////////////////////////////////////////////////

function MessageDlgEx(const aCaption: String; const Msg: string;
  DlgType: TMsgDlgExType; Buttons: TMsgDlgExButtons;
  DefaultButton, CancelButton: TMsgDlgExBtn; HelpCtx: Longint): Integer;
var i: Integer;
    Btn: TButton;
begin
  with CreateMessageDialogEx(Msg, DlgType, Buttons, DefaultButton, CancelButton) do
    try
      If aCaption <> '' Then
        Caption := aCaption;
      HelpContext := HelpCtx;
      for i := 0 to ComponentCount-1 do begin
        if Components[i] is TButton then begin
          btn := TButton(Components[i]);
          btn.Default := btn.ModalResult = ExModalResults[DefaultButton];
          if btn.Default then
            ActiveControl := Btn;
        end;
      end;
      Result := ShowModal;
    finally
      Free;
    end;
end;

end.
