{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

Initial Autor   : Marcus Fuchs

Last Modified at: 26. January 2003
Last Modified by: Marcus Fuchs

You may retrieve the latest version of this file at the MFLib home page,
located at http://www.mfhome.net/mflib/
-------------------------------------------------------------------------------}

unit MFMiscCtrl;

interface

// remove the period to enable Debug settings (see MFLib.inc)
{.$DEFINE DEBUG}

{$I MFLib.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,     { VCL   }
  Dialogs;                                                             { VCL   }

const
  cMinHandledSize = 7;

type
  NaturalNumber = 1..High(Integer);

  TCanResizeEvent = procedure(Sender: TObject; var NewSize: Integer;
    var Accept: Boolean) of object;

  TResizeStyle = (rsNone, rsLine, rsUpdate, rsPattern);
  THandleAlign = (haLeft, haCenter, haRight);
  TClickOrientation = (coLeft, coRight, coUp, coDown);
  TDirection = (diLeft, diRight, diUp, diDown);
  TMFHandleWidthType = (hwtPixels, hwtPercentage); // MF

  TMFSplitter = class(TGraphicControl)
  private
    ctop, cbottom,
    cleft, cright: Integer;
    MouseX, MouseY: Integer;
    bmLeft, bmRight, bmUp, bmDown: TBitmap;
    ctrl1, ctrl2: TControl;
    FOldControlSize: Integer;

    FSplitCursor: TCursor;
    FActiveControl: TWinControl;
    FHandled: Boolean;
    FHandleSize: NaturalNumber;
    FHandleWidthType: TMFHandleWidthType; // MF
    FHandleAlign: THandleAlign;
    FBrush: TBrush;
    FControl: TControl;
    FDownPos: TPoint;
    FLineDC: HDC;
    FLineVisible: Boolean;
    FMinSize: NaturalNumber;
    FMaxSize: Integer;
    FNewSize: Integer;
    FOldKeyDown: TKeyEvent;
    FOldSize: Integer;
    FPrevBrush: HBrush;
    FResizeStyle: TResizeStyle;
    FSplit: Integer;

    FOnCanResize: TCanResizeEvent;
    FOnMoved: TNotifyEvent;
    FOnPaint: TNotifyEvent;
    FClickOrientation: TClickOrientation;
    procedure AllocateLineDC;
    procedure CalcSplitSize(X, Y: Integer; var NewSize, Split: Integer);
    procedure DrawLine;
    function FindControl(aDirection: TDirection): TControl;
    procedure FocusKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure ReleaseLineDC;
    procedure SetHandled(Value: Boolean);
    procedure UpdateControlSize;
    procedure UpdateSize(X, Y: Integer);
    procedure SetClickOrientation(const Value: TClickOrientation);
    function GetAlign: TAlign;
    procedure SetAlign(const Value: TAlign);
    procedure SetHandleSize(const Value: NaturalNumber);
    procedure SetHandleWidthType(const Value: TMFHandleWidthType);
    procedure SetHandleAlign(const Value: THandleAlign);
    function GetWidth: Integer;
    procedure SetWidth(const Value: Integer);
    procedure SetHeight(Value: Integer);
    function GetHeight: Integer;
  protected
    function CanResize(var NewSize: Integer): Boolean; {reintroduce; }virtual;
    function DoCanResize(var NewSize: Integer): Boolean; virtual;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Paint; override;
    procedure RequestAlign; {$IFDEF DELPHI4_UP}override;{$ENDIF}
    procedure StopSizing; dynamic;

    function MouseInHandle: Boolean;

    // protected -> no one has to know
    property ClickOrientation: TClickOrientation read FClickOrientation write SetClickOrientation;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;
  published
    property Align: TAlign read GetAlign write SetAlign default alLeft;
    property Handled: Boolean read FHandled write SetHandled default True;
    property HandleSize: NaturalNumber read FHandleSize write SetHandleSize default 80;
    property HandleWidthType: TMFHandleWidthType read FHandleWidthType write SetHandleWidthType default hwtPixels;
    property HandleAlign: THandleAlign read FHandleAlign write SetHandleAlign default haCenter;
    property Color;
  {$IFDEF DELPHI4_UP}
    property Constraints;
  {$ENDIF}
    property MinSize: NaturalNumber read FMinSize write FMinSize default 30;
    property ParentColor;
    property ResizeStyle: TResizeStyle read FResizeStyle write FResizeStyle default rsPattern;
    property Visible;
    property Width: Integer read GetWidth write SetWidth;

    property OnCanResize: TCanResizeEvent read FOnCanResize write FOnCanResize;
    property OnMoved: TNotifyEvent read FOnMoved write FOnMoved;
    property OnPaint: TNotifyEvent read FOnPaint write FOnPaint;
    property Height: Integer read GetHeight write SetHeight;
  end;


implementation

{$R ..\Resources\MFSplitter.res}
{$R ..\Resources\MFSplitter.dcr}

type
  TWinControlAccess = class(TWinControl);

constructor TMFSplitter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Align := alLeft;
  Width := 7;
  Cursor := crHSplit;
  FHandled := True;
  FHandleSize := 80;
  FHandleWidthType := hwtPixels;
  FHandleAlign := haCenter;
  FMinSize := 30;
  FResizeStyle := rsPattern;
  FOldSize := -1;

  bmLeft     := TBitmap.Create;
  bmRight    := TBitmap.Create;
  bmUp       := TBitmap.Create;
  bmDown     := TBitmap.Create;

  bmLeft.LoadFromResourceName(HInstance, 'UPCLICK_LEFT');
  bmRight.LoadFromResourceName(HInstance, 'UPCLICK_RIGHT');
  bmUp.LoadFromResourceName(HInstance, 'UPCLICK_UP');
  bmDown.LoadFromResourceName(HInstance, 'UPCLICK_DOWN');
end;

destructor TMFSplitter.Destroy;
begin
  FBrush.Free;
  bmLeft.Free;
  bmRight.Free;
  bmUp.Free;
  bmDown.Free;
  inherited Destroy;
end;

procedure TMFSplitter.AllocateLineDC;
begin
  FLineDC := GetDCEx(Parent.Handle, 0, DCX_CACHE or DCX_CLIPSIBLINGS
    or DCX_LOCKWINDOWUPDATE);
  {if ResizeStyle = rsPattern then begin
    if FBrush = nil then begin
      FBrush := TBrush.Create;
     // FBrush.Bitmap := AllocPatternBitmap(clBlack, clWhite);
    end;
    FPrevBrush := SelectObject(FLineDC, FBrush.Handle);
  end;}
end;

procedure TMFSplitter.DrawLine;
var
  P: TPoint;
begin
  FLineVisible := not FLineVisible;
  P := Point(Left, Top);
  if Align in [alLeft, alRight] then
    P.X := Left + FSplit
  else
    P.Y := Top + FSplit;
  with P do PatBlt(FLineDC, X, Y, Width, Height, PATINVERT);
end;

procedure TMFSplitter.ReleaseLineDC;
begin
  if FPrevBrush <> 0 then
    SelectObject(FLineDC, FPrevBrush);
  ReleaseDC(Parent.Handle, FLineDC);
  if FBrush <> nil then begin
    FBrush.Free;
    FBrush := nil;
  end;
end;

function TMFSplitter.FindControl(aDirection: TDirection): TControl;
var
  P: TPoint;
  I: Integer;
  R: TRect;
begin
  P := Point(Left, Top);
  case aDirection of
    diLeft : Dec(P.x);
    diRight: Inc(P.x, Width);
    diUp   : Dec(P.y);
    diDown : Inc(P.y, Height);
  end;
{  case Align of
    alLeft: Dec(P.X);
    alRight: Inc(P.X, Width);
    alTop: Dec(P.Y);
    alBottom: Inc(P.Y, Height);
  else
    Exit;
  end;}
  for I := 0 to Parent.ControlCount - 1 do begin
    Result := Parent.Controls[I];
    if Result.Visible and Result.Enabled then begin
      R := Result.BoundsRect;
      if (R.Right - R.Left) = 0 then
        if Align in [alTop, alLeft] then
          Dec(R.Left)
        else
          Inc(R.Right);
      if (R.Bottom - R.Top) = 0 then
        if Align in [alTop, alLeft] then
          Dec(R.Top)
        else
          Inc(R.Bottom);
      if PtInRect(R, P) then
        Exit;
    end;
  end;
  Result := nil;
end;

procedure TMFSplitter.RequestAlign;
begin
{$IFDEF DELPHI4_UP}
  inherited RequestAlign;
{$ENDIF}
  if (Cursor <> crVSplit) and (Cursor <> crHSplit) then
    Exit;
  if Align in [alBottom, alTop] then
    Cursor := crVSplit
  else
    Cursor := crHSplit;
end;

procedure TMFSplitter.Paint;

  function CalcHandleSize: NaturalNumber; // MF
  var tmp: Single;
  begin
    if FHandleWidthType = hwtPixels then
      Result := FHandleSize
    else begin
      if Align in [alLeft, alRight] then
        if FHandleSize = 100 then
          Result := Parent.ClientHeight
        else begin
          tmp := Parent.ClientHeight / 100 * FHandleSize;
          if tmp < 1 then
            Result := 1
          else
            Result := Trunc(tmp)
        end
      else
        if FHandleSize = 100 then
          Result := Parent.ClientWidth
        else begin
          tmp := Parent.ClientWidth / 100 * FHandleSize;
          if tmp < 1 then
            Result := 1
          else
            Result := Trunc(tmp)
        end;
    end;
  end;

const
  XorColor = $00FFD8CE;
var
  R: TRect;
  bm: TBitmap;
  RealHandleSize: NaturalNumber;
begin
  R := ClientRect;
  with Canvas do begin
    Pen.Color := Color;
    Pen.Style := psSolid;
    Brush.Color := Color;
    Pen.Mode := pmCopy;
    FillRect(ClientRect);
    Pen.Color := clBtnShadow;
  end;

  RealHandleSize := CalcHandleSize; // MF

  if Align in [alLeft, alRight] then begin
    case HandleAlign of
      haLeft   : begin
                   ctop := 0; // MF
                   cbottom := ctop + RealHandleSize;
                 end;
      haCenter : begin
                   ctop := (ClientHeight div 2) - (RealHandleSize div 2); // MF
                   cbottom := ctop + RealHandleSize;
                 end;
      haRight  : begin
                   ctop := ClientHeight - RealHandleSize; // MF
                   cbottom := ctop + RealHandleSize;
                 end;
    end;
  end
  else begin
     case HandleAlign of
       haLeft   : begin
                    cleft := 0; // MF
                    cright := cleft + RealHandleSize;
                  end;
       haCenter : begin
                    cleft := (ClientWidth div 2) - (RealHandleSize div 2); // MF
                    cright := cleft + RealHandleSize;
                  end;
       haRight  : begin
                    cleft := ClientWidth - RealHandleSize; // MF
                    cright := cleft + RealHandleSize;
                  end;
     end;
   end;

  if Handled then
    case ClickOrientation of
      coUp,
      coDown  : begin
                  with Canvas do begin
                    Polygon([Point(cright - 1, 0), Point(cleft, 0),
                             Point(cleft, Height - 1), Point(cright - 1, Height - 1),
                             Point(cright - 1, 0)]);
                    Pen.Color := clBtnHighlight;
                    PolyLine([Point(cleft + 1, Height - 2), Point(cleft + 1, 1), Point(cright - 1, 1)]);

                    // driehoekjes
                    if ClickOrientation = coUp then
                      bm := bmUp
                    else
                      bm := bmDown;
                    Draw(cleft + 4, 2, bm);
                    Draw(cright - bm.Width - 4, 2, bm);
                  end;
                end;
      coLeft,
      coRight : begin
                  with Canvas do begin
                    PolyLine([Point(0, cbottom - 1), Point(0, ctop), Point(Width - 1, ctop),
                              Point(Width - 1, cbottom - 1), Point(0, cbottom - 1)]);
                    Pen.Color := clBtnHighlight;
                    PolyLine([Point(Width - 2, ctop + 1), Point(1, ctop + 1), Point(1, cbottom - 1)]);
                    if ClickOrientation = coLeft then
                      bm := bmLeft
                    else
                      bm := bmRight;
                    // driehoekje
                    Draw(2, ctop + 4, bm);
                    Draw(2, cbottom - bm.Height - 4, bm);
                  end;
                end;
    end;

  if csDesigning in ComponentState then
    { Draw outline }
    with Canvas do begin
      Pen.Style := psDot;
      Pen.Mode := pmXor;
      Pen.Color := XorColor;
      Brush.Style := bsClear;
      Rectangle(0, 0, ClientWidth, ClientHeight);
    end;
  if Assigned(FOnPaint) then
    FOnPaint(Self);
end;

function TMFSplitter.DoCanResize(var NewSize: Integer): Boolean;
begin
  Result := CanResize(NewSize);
  if Result and (NewSize <= MinSize) then
    NewSize := MinSize;// MT this was the bug in the TSplitter component in Delphi 4 (NewSize := 0)
end;

function TMFSplitter.CanResize(var NewSize: Integer): Boolean;
begin
  Result := True;
  if Assigned(FOnCanResize) then
    FOnCanResize(Self, NewSize, Result);
end;

procedure TMFSplitter.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  I: Integer;
begin
  inherited MouseDown(Button, Shift, X, Y);
  if Button = mbLeft then begin
    case Align of
      alLeft   : FControl := ctrl1;
      alRight  : FControl := ctrl2;
      alTop    : FControl := ctrl1;
      alBottom : FControl := ctrl2;
    end;
    FDownPos := Point(X, Y);
    if Assigned(FControl) then begin
      if Align in [alLeft, alRight] then begin
        FMaxSize := Parent.ClientWidth - FMinSize;
        for I := 0 to Parent.ControlCount - 1 do
          with Parent.Controls[I] do
            if Align in [alLeft, alRight] then
              Dec(FMaxSize, Width);
        Inc(FMaxSize, FControl.Width);
      end
      else begin
        FMaxSize := Parent.ClientHeight - FMinSize;
        for I := 0 to Parent.ControlCount - 1 do
          with Parent.Controls[I] do
            if Align in [alTop, alBottom] then Dec(FMaxSize, Height);
        Inc(FMaxSize, FControl.Height);
      end;
      UpdateSize(X, Y);
      AllocateLineDC;
      with ValidParentForm(Self) do
        if ActiveControl <> nil then begin
          FActiveControl := ActiveControl;
          FOldKeyDown := TWinControlAccess(FActiveControl).OnKeyDown;
          TWinControlAccess(FActiveControl).OnKeyDown := FocusKeyDown;
        end;
      if ResizeStyle in [rsLine, rsPattern] then
        DrawLine;
    end;
  end;
end;

procedure TMFSplitter.UpdateControlSize;
begin
  if FNewSize <> FOldSize then begin
    case Align of
      alLeft: begin
                FControl.Width := FNewSize;
                Left := FControl.Width + 1;
              end;
      alTop: begin
               FControl.Height := FNewSize;
               Top := FControl.Height + 1;
             end;
      alRight:
        begin
          Parent.DisableAlign;
          try
            FControl.Left := FControl.Left + (FControl.Width - FNewSize);
            FControl.Width := FNewSize;
          finally
            Parent.EnableAlign;
          end;
          {$IFNDEF DELPHI4_UP}
          if FControl.Width <> 0 then begin
            Left := FControl.Left;
            FControl.Left := Left + 1;
          end;
          {$ENDIF}
        end;
      alBottom:
        begin
          Parent.DisableAlign;
          try
            FControl.Top := FControl.Top + (FControl.Height - FNewSize);
            FControl.Height := FNewSize;
          finally
            Parent.EnableAlign;
          end;
          {$IFNDEF DELPHI4_UP}
          if FControl.Height <> 0 then begin
             Top := FControl.Top;
             FControl.Top := Top + 1;
           end;
          {$ENDIF}
        end;
    end;
    Update;
    if Assigned(FOnMoved) then
      FOnMoved(Self);
    if FOldSize = 0 then
      case Align of
        alLeft    : ClickOrientation := coLeft;
        alBottom  : ClickOrientation := coDown;
        alRight   : ClickOrientation := coRight;
        alTop     : ClickOrientation := coUp;
      end;
    FOldSize := FNewSize;
  end;
end;

procedure TMFSplitter.CalcSplitSize(X, Y: Integer; var NewSize, Split: Integer);
var
  S: Integer;
begin
  if Align in [alLeft, alRight] then
    Split := X - FDownPos.X
  else
    Split := Y - FDownPos.Y;
  S := 0;
  case Align of
    alLeft: S := FControl.Width + Split;
    alRight: S := FControl.Width - Split;
    alTop: S := FControl.Height + Split;
    alBottom: S := FControl.Height - Split;
  end;
  NewSize := S;
  if S < FMinSize then
    NewSize := FMinSize
  else if S > FMaxSize then
    NewSize := FMaxSize;
  if S <> NewSize then begin
    if Align in [alRight, alBottom] then
      S := S - NewSize
    else
      S := NewSize - S;
    Inc(Split, S);
  end;
end;

procedure TMFSplitter.UpdateSize(X, Y: Integer);
begin
  CalcSplitSize(X, Y, FNewSize, FSplit);
end;

procedure TMFSplitter.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  NewSize, Split: Integer;
begin
  inherited;
  if MouseInHandle then
    Cursor := crDefault
  else
    Cursor := FSplitCursor;
  MouseX := x;
  MouseY := y;
  if (ssLeft in Shift) and Assigned(FControl) then begin
    CalcSplitSize(X, Y, NewSize, Split);
    if DoCanResize(NewSize) then begin
      if ResizeStyle in [rsLine, rsPattern] then
        DrawLine;
      FNewSize := NewSize;
      FSplit := Split;
      if ResizeStyle = rsUpdate then
        UpdateControlSize;
      if ResizeStyle in [rsLine, rsPattern] then
        DrawLine;
    end;
  end;
end;

procedure TMFSplitter.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;
  if Assigned(FControl) then begin
    if ResizeStyle in [rsLine, rsPattern] then
      DrawLine;
    if MouseInHandle then begin
      if FOldSize = -1 then begin // first time
        FNewSize := 0;
        if ClickOrientation in [coUp, coDown] then
          FOldControlSize := FControl.Height
        else
          FOldControlSize := FControl.Width;
      end
      else
        if FOldSize > 0 then begin
          FNewSize := 0; //
          FOldControlSize := FOldSize;
        end
        else begin
          FNewSize := FOldControlSize;
        end;
      case ClickOrientation of
        coUp    : ClickOrientation := coDown;
        coDown  : ClickOrientation := coUp;
        coLeft  : ClickOrientation := coRight;
        coRight : ClickOrientation := coLeft;
      end;
    end;
    UpdateControlSize;
    StopSizing;
  end;
end;

procedure TMFSplitter.FocusKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_ESCAPE then
    StopSizing
  else
    if Assigned(FOldKeyDown) then
      FOldKeyDown(Sender, Key, Shift);
end;

procedure TMFSplitter.SetHandled(Value: Boolean);
begin
  FHandled := Value;
  case Align of
    alLeft, alRight : if Width < cMinHandledSize
                      then Width := cMinHandledSize;
    alBottom, alTop : if Height < cMinHandledSize
                      then Height := cMinHandledSize;
  end;
  RePaint;
end;

procedure TMFSplitter.StopSizing;
begin
  if Assigned(FControl) then begin
    if FLineVisible then
      DrawLine;
    FControl := nil;
    ReleaseLineDC;
    if Assigned(FActiveControl) then begin
      TWinControlAccess(FActiveControl).OnKeyDown := FOldKeyDown;
      FActiveControl := nil;
    end;
  end;
  if Assigned(FOnMoved) then
    FOnMoved(Self);
end;

procedure TMFSplitter.SetClickOrientation(const Value: TClickOrientation);
begin
  FClickOrientation := Value;
  Invalidate;
end;

procedure TMFSplitter.Loaded;
begin
  inherited Loaded;
  FSplitCursor := Cursor;
  if Align in [alLeft, alRight] then begin
    ctrl1 := FindControl(diLeft);
    ctrl2 := FindControl(diRight);
  end;
  if Align in [alTop, alBottom] then begin
    ctrl1 := FindControl(diUp);
    ctrl2 := FindControl(diDown);
  end;
end;

function TMFSplitter.MouseInHandle: Boolean;
begin
  if Align in [alLeft, alRight] then
    Result := Handled and PtInRect(Rect(0, ctop, Width, cbottom), Point(MouseX, MouseY))
  else
    Result := Handled and PtInRect(Rect(cleft, 0, cright, Height), Point(MouseX, MouseY));
end;

function TMFSplitter.GetAlign: TAlign;
begin
  Result := inherited Align;
end;

procedure TMFSplitter.SetAlign(const Value: TAlign);
var aValue: TClickOrientation;
begin
  if Value in [alNone, alClient] then
    raise Exception.Create('Value not allowed for ' + ClassName + ' component.');
  inherited Align := Value;
  aValue := coLeft;
  case Align of
    alLeft  : aValue := coLeft;
    alRight : aValue := coRight;
    alTop   : aValue := coUp;
    alBottom: aValue := coDown;
  end;
  ClickOrientation := aValue;
end;

procedure TMFSplitter.SetHandleSize(const Value: NaturalNumber);
begin
  if Value <> FHandleSize then begin // MF
    FHandleSize := Value;
    Invalidate;
  end;
end;

procedure TMFSplitter.SetHandleAlign(const Value: THandleAlign);
begin
  if Value <> FHandleAlign then begin // MF
    FHandleAlign := Value;
    Invalidate;
  end;
end;

procedure TMFSplitter.SetHandleWidthType(const Value: TMFHandleWidthType);
begin
  if Value <> FHandleWidthType then begin
    FHandleWidthType := Value;
    if (FHandleWidthType = hwtPercentage) and (FHandleSize > 100) then
      FHandleSize := 100;
    if FHandled then
      Invalidate;
  end;
end;

function TMFSplitter.GetWidth: Integer;
begin
  Result := inherited Width;
end;

procedure TMFSplitter.SetWidth(const Value: Integer);
begin
  if Handled and (Align in [alLeft, alRight]) and (Value < cMinHandledSize) then
    raise Exception.Create('Min. width for a handled TMFSplitter component is ' + IntToStr(cMinHandledSize));
  inherited Width := Value;
end;

procedure TMFSplitter.SetHeight(Value: Integer);
begin
  if Handled and (Align in [alTop, alBottom]) and (Value < cMinHandledSize) then
    raise Exception.Create('Min. height for a handled TMFSplitter component is ' + IntToStr(cMinHandledSize));
  inherited Height := Value;
end;

function TMFSplitter.GetHeight: Integer;
begin
  Result := inherited Height;
end;

end.
