{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY of ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

Initial Autor   : Marcus Fuchs

Last Modified at: 05. February 2003
Last Modified by: Marcus Fuchs

You may retrieve the latest version of this file at the MFLib home page,
located at http://www.mfhome.net/mflib/
-------------------------------------------------------------------------------}

unit MFRoutines;

interface { ------------------------------------------------------ interface - }

// remove the period to enable Debug settings (see MFLib.inc)
{.$DEFINE DEBUG}

{$I MFLib.inc}

uses
  Windows, Classes, SysUtils, Forms, ShellApi, ShlObj, Registry,       { VCL   }
  StdCtrls, Messages,                                                  { VCL   }
  MFTypes;                                                             { MFLib }

const
  CRLF = #13+#10;
{ Summary: New line in Windows. }
{ Description: CarriageReturn and LineFeed, used on Windows for a new line. }

type
  TFileSearchOption = (fsoOnlyGivenDir, fsoWithinSubDirs,
     fsoAbsolut, fsoRelativ);
  TFileSearchOptions = set of TFileSearchOption;

type
  TSystemPath = (spWindows, spSystem, spTemp, spDesktop, spPersonal, spPrograms,
     spRecent, spSendTo, spStartMenu, spStartUp, spTemplates, spFavorites,
     spAppData);

type
  TFileInfoStr = (fiCompanyName, fiFileDescription, fiFileVersion,
    fiInternalName, fiLegalCopyright, fiLegalTrademarks, fiOriginalFilename,
    fiProductName, fiProductVersion, fiComments,
    fiHomepage, fiMail);

const
  FileInfoStr: Array[TFileInfoStr] of String = ('CompanyName', 'FileDescription',
    'FileVersion', 'InternalName', 'LegalCopyright', 'LegalTrademarks',
    'OriginalFilename', 'ProductName', 'ProductVersion', 'Comments',
    'Homepage', 'E-mail');

function StreamReadln(Source: TStream): String; { Thanks to Marian Aldenhvel }
{ Summary: Reads a single line from a stream }

function IndexOfValue(const StringList: TStringList;
   const Value: String): Integer;
{ Note: Only returns the first match. }

procedure GetSubdirs(Dir: String; var SubDirs: TStringList;
   const SearchOptions: TFileSearchOptions);
{ Summary: Gives all Subdirectories from the given directory. }
{ Description:  }
{ See Also: TFileSearchOptions }

function DelDir(Dir: String): Boolean; { Found in EDH (Easy Delphi Helper) }
{ Summary: Deletes the given Directory. }
{ Description: Use DelDir if you want to delete the hole directory, including
  all files and subfolders. }

function RelPathToRealPath2(AnchorPath: String; RelPath: String): String;
{ AnchorPath is only the part befor RelPath
  so if one String is empty it doesn't matter which one

  Thies are all treaten the same way:
  "\.\Programme\.\" = "\.\Programme\." = "\.\Programme\" = "\.\Programme"
   = ".\Programme" = "\Programme" = "Programme" = "Programme\" = "Programme\."
   = "Programme\.\" = "\Programme\.\" = ".\Programme\.\" = "\.\Programme\.\"
}
{ Note: This will be renamed to RelPathToRealPath, in futur versions. }

// procedure DelComments(var str: String; const OneLineComment: String;
//   const MultiLineComment: Array[1..2] of String): Boolean;
{ Deletes all after (including) the OneLineComment in one line and
  Deletes all between (including) the MultiLineComments }

procedure DelComments(var str: String);
{ Summary: Delete all Delphicomments. }
{ Description: Deletes all Delphicomments in the given String. }

function GetSystemPath(const Path: TSystemPath): String; { Found in a PC Magazin }
{ Summary: Returns the Path to a SystemPath. }

procedure MarkText(CustomMemo: TCustomMemo; Start, Len: Integer);

function GetFirst(CustomMemo: TCustomMemo; Range: TSearchRange): Integer;
{ Description: Returns the Index of the first Char, depending on Range }

function GetLast(CustomMemo: TCustomMemo; Range: TSearchRange): Integer;
{ Description: Returns the Index of the last Char, depending on Range }

function StringSearch(const Source, SearchStr: String;
   const First, Last: Integer; Flags: TRichSearchTypes): Integer;
{ Description: Returns the first position (zero-based) of SearchStr in Source }

function StringReplace(const S, OldPattern, NewPattern: String;
   Flags: TReplaceFlags {$IFDEF SUPPORTS_DEFAULTPARAMS}= [rfReplaceAll]{$ENDIF})
   : String; {$IFDEF DELPHI4_UP}overload;{$ENDIF} { Thanks to Bjrge Sther }

function GetExeForProtocol(URL: String): String; { Taken from SRUtils.pas }

procedure GetVersionInfo(const aFileName: String; aVersionList: TStrings);
{ Description: aVersionList will get all FileInfoStrings found in the given
  File. Empty FileInfoStrings will result in an empty String at his Index. }


implementation { -------------------------------------------- implementation - }

uses
  MFStrRes;

////////////////////////////////////////////////////////////////////////////////

function StreamReadln(Source: TStream): String;
  function StreamReadChar(Source: TStream): Char;
  begin
    if Source.Read(Result, Sizeof(char)) = 0 then
      Result := #26; { end of file }
  end;
var bufsize, charsRead: Integer;
    ch: Char;
begin
  bufsize := 255;
  charsRead := 0;
  SetLength(Result, bufsize);
  repeat
    ch := StreamReadChar(Source);
    case ch of
      #13: if StreamReadChar(Source) <> #10 then Source.Seek(-1, soFromCurrent);
      #10: ch := #13;
      #26: begin ch := #13; Source.Seek(0, soFromEnd); end;
      else begin
        inc(charsRead);
        if charsRead > bufsize then begin
          inc(bufsize, 128);
          SetLength(Result, bufsize);
        end;
        Result[charsRead] := ch;
      end;{else}
    end;{case}
  until (ch = #13);
  SetLength(Result, charsRead);
end;

////////////////////////////////////////////////////////////////////////////////

function IndexOfValue(const StringList: TStringList;
   const Value: String): Integer;
var P: Integer;
    S: String;
begin
  if (StringList <> nil) and (StringList.Count > 0) then
    For Result := 0 to StringList.Count - 1 do begin
      S := StringList.Strings[Result];
      P := AnsiPos('=', S);
      if (P <> 0) and (AnsiCompareText(Copy(S, P+1, Length(S)), Value) = 0) then
        Exit;
    end;
  Result := -1;
end;

////////////////////////////////////////////////////////////////////////////////

procedure GetRelSubdirs(StartDir: String; RelDir: String;
   var RelSubDirs: TStringList; const SearchOptions: TFileSearchOptions);
{ if SearchOptions = [] then SearchOptions = [fsoOnlyGivenDir, fsoAbsolut] }
{ if SearchOptions = [fsoWithinSubdirs] then SearchOptions =
    [fsoWithinSubdirs, fsoAbsolut] }
{ if SearchOptions = [fsoRelativ] then SearchOptions =
    [fsoOnlyGivenDir, fsoRelativ] }
var SR     : TSearchRec;
    IsFound: Boolean;
begin
  if StartDir[Length(StartDir)] <> '\' then
    StartDir := StartDir + '\';
  if (Length(RelDir) > 0) and (RelDir[Length(RelDir)] <> '\') then
    RelDir := RelDir + '\';
  IsFound := FindFirst(StartDir+RelDir+'*', faDirectory, SR) = 0;
  try
    while IsFound do begin
      if (SR.Attr and faDirectory > 0) and
         (Copy(SR.Name, 1, 1) <> '.') then begin
        RelSubDirs.Add(RelDir+SR.Name);
        if fsoWithinSubDirs in SearchOptions then
          GetRelSubdirs(StartDir, RelDir+SR.Name, RelSubDirs, SearchOptions);
        Application.ProcessMessages;
      end;
      IsFound := FindNext(SR) = 0;
    end;{while}
  finally
    FindClose(SR);
  end;
end;

////////////////////////////////////////////////////////////////////////////////

procedure GetSubdirs(Dir: String; var SubDirs: TStringList;
   const SearchOptions: TFileSearchOptions);
{ if SearchOptions = [] then SearchOptions = [fsoOnlyGivenDir, fsoAbsolut] }
{ if SearchOptions = [fsoWithinSubdirs] then SearchOptions =
    [fsoWithinSubdirs, fsoAbsolut] }
{ if SearchOptions = [fsoRelativ] then SearchOptions =
    [fsoOnlyGivenDir, fsoRelativ] }
var SR       : TSearchRec;
    IsFound  : Boolean;
    RelSubDir: String;
begin
  if Dir[Length(Dir)] <> '\' then
    Dir := Dir + '\';
  if (fsoRelativ in SearchOptions) then begin
    RelSubDir := '';
    GetRelSubDirs(Dir, RelSubDir, SubDirs, SearchOptions);
    Exit
  end{ if (fsoRelativ in SearchOptions) then }
  else begin
    IsFound := FindFirst(Dir+'*', faDirectory, SR) = 0;
    try
      while IsFound do begin
        if (SR.Attr and faDirectory > 0) and
           (Copy(SR.Name, 1, 1) <> '.') then begin
          SubDirs.Add(Dir+SR.Name);
          if fsoWithinSubDirs in SearchOptions then
            GetSubdirs(Dir+SR.Name, SubDirs, SearchOptions);
          Application.ProcessMessages;
        end;
        IsFound := FindNext(SR) = 0;
      end;{ while }
    finally
      FindClose(SR);
    end;
  end;{ else }
end;

////////////////////////////////////////////////////////////////////////////////

function DelDir(Dir: String): Boolean;
var FileOption: TSHFileOpStruct;
begin
  ZeroMemory(@FileOption, SizeOf(FileOption));
  FileOption.wFunc := FO_DELETE;
  FileOption.fFlags := FOF_SILENT or FOF_NOCONFIRMATION;

  if Dir[Length(Dir)] = '\' then
    Delete(Dir, Length(Dir), 1);

  FileOption.pFrom := PChar(Dir + #0);
  Result := (SHFileOperation(FileOption) = 0);
end;

////////////////////////////////////////////////////////////////////////////////

function RelPathToRealPath2(AnchorPath: String; RelPath: String): String;
var thisdirpos, abovedirpos: Integer;
    i: Integer;
const thisdirstr = '.\';
      abovedirstr = '..\';
begin
  if (RelPath <> '') then
    if RelPath[1] = '\' then
      Delete(RelPath, 1, 1);

  if AnchorPath <> '' then begin
    if AnchorPath[Length(AnchorPath)] <> '\' then
      RelPath := AnchorPath+'\'+RelPath
    else
      RelPath := AnchorPath+RelPath;

    if RelPath[1] = '\' then
      Delete(RelPath, 1, 1);
  end;

  if RelPath[Length(RelPath)] <> '\' then
    RelPath := RelPath+'\';

  thisdirpos := Pos('.\', RelPath);
  if thisdirpos > 0 then begin
    abovedirpos := Pos(abovedirstr, RelPath);
    if abovedirpos > 0 then begin
      if RelPath[abovedirpos-1] = '\' then begin
        { 'C:\Programme\..\MFS\Prog1' -> 'C:\MFS\Prog1' }
        For i := abovedirpos-2 downto 0 do begin
          if RelPath[i] = '\' then begin
            Delete(RelPath, i, abovedirpos-1+Length(abovedirstr)-i);
            Break;
          end
          else
            if RelPath[i] = ':' then begin
              { C:\..\MFS\Prog1\ -> - }
              Result := '';
              Exit;
            end
            else
              if i = 0 then
                { Programm\..\MFS\Prog1\ -> MFS\Prog1\ }
                Delete(RelPath, i+1, abovedirpos-1+Length(abovedirstr)-i);
        end;
        Result := RelPathToRealPath2('', RelPath);
      end
      else
        { '..\MFS\Prog1' -> - }
        Result := '';
    end
    else begin
      if RelPath[thisdirpos-1] = '\' then begin
        { 'C:\Programme\.\MFS\Prog1' -> 'C:\Programme\MFS\Prog1' }
        Delete(RelPath, thisdirpos, Length(thisdirstr));
        Result := RelPathToRealPath2('', RelPath);
      end
      else
        { '.\MFS\Prog1' -> 'MFS\Prog1' }
//        Result := Copy(RelPath, thisdirpos+Length(thisdirstr), Length(RelPath));
        { to also get .\MFS\Prog\.\ }
        Result := RelPathToRealPath2('', Copy(RelPath,
                     thisdirpos+Length(thisdirstr), Length(RelPath)));
    end;
  end
  else
    Result := RelPath;
end;

////////////////////////////////////////////////////////////////////////////////

procedure DelComments(var str: String);
var start, start1, start2,
    stop, stop1, stop2, stop3: Integer;
    temp: String;
begin
  start := Pos('//', str);
  while start >= 1 do begin
    temp := copy(str, start, Length(str));
    stop1 := Pos(CRLF, temp); // Zeilenende/ Ende des Kommentares
    stop2 := Pos('}', temp);  // Ende eines anderen Kommentares
    stop3 := Pos('*)', temp); // Ende eines anderen Kommentares
    if stop1 > 0 then
      if stop2 > 0 then
        if stop3 > 0 then
          if stop2 > stop3 then
            if stop1 > stop3 then
              stop := stop3
            else
              stop := stop1
          else
            if stop1 > stop2 then
              stop := stop2
            else
              stop := stop1
        else
          if stop1 > stop2 then
            stop := stop2
          else
            stop := stop1
      else
        if stop3 > 0 then
          stop := stop3
        else
          stop := stop1
    else { Dateiende }
      if stop2 > 0 then
        if stop3 > 0 then
          if stop2 > stop3 then
            stop := stop3
          else
            stop := stop2
        else
          stop := stop2
      else
        if stop3 > 0 then
          stop := stop3
        else
          stop := Length(temp);
    delete(str, start, stop-1);
    start := Pos('//', str);
  end;

  start1 := Pos('{', str);
  start2 := Pos('(*', str);
  while (start1 > 0) or (start2 > 0) do begin
    if start1 > 0 then
      if start2 > 0 then
        if start1 > start2 then begin
          temp := copy(str, start2, Length(str));
          stop := Pos('*)', temp)+1;
          if stop = 0 then
            Exit;
          delete(str, start2, stop);
        end
        else begin
          temp := copy(str, start1, Length(str));
          stop := Pos('}', temp);
          if stop = 0 then
            Exit;
          delete(str, start1, stop);
        end
      else begin
        temp := copy(str, start1, Length(str));
        stop := Pos('}', temp);
        if stop = 0 then
          Break;
        delete(str, start1, stop);
      end
    else
      if start2 > 0 then begin
        temp := copy(str, start2, Length(str));
        stop := Pos('*)', temp)+1;
        if stop = 0 then
          Exit;
        delete(str, start2, stop);
      end;
    start1 := Pos('{', str);
    start2 := Pos('(*', str);
  end;
end;

////////////////////////////////////////////////////////////////////////////////

function GetSystemPath(const Path: TSystemPath): String;
var CSIDL: Integer;
    PIDL: PItemIDList;
begin
  case Path of
    spWindows: begin
      SetLength(Result, MAX_PATH);
      SetLength(Result, GetWindowsDirectory(PChar(Result), MAX_PATH));
    end;
    spSystem: begin
      SetLength(Result, MAX_PATH);
      SetLength(Result, GetSystemDirectory(PChar(Result), MAX_PATH));
    end;
    spTemp: begin
      SetLength(Result, MAX_PATH);
      SetLength(Result, GetTempPath(MAX_PATH, PChar(Result)));
    end;
  else
    begin
      // Abfrage ber SHGetSpecialFolderLocation
      case Path of
        spDesktop:   CSIDL := CSIDL_DESKTOPDIRECTORY;
        spPersonal:  CSIDL := CSIDL_PERSONAL;
        spPrograms:  CSIDL := CSIDL_PROGRAMS;
        spRecent:    CSIDL := CSIDL_RECENT;
        spSendTo:    CSIDL := CSIDL_SENDTO;
        spStartMenu: CSIDL := CSIDL_STARTMENU;
        spStartUp:   CSIDL := CSIDL_STARTUP;
        spTemplates: CSIDL := CSIDL_TEMPLATES;
        spFavorites: CSIDL := CSIDL_FAVORITES;
        spAppData:   CSIDL := CSIDL_APPDATA;
      else
        CSIDL := 0;
      end;

      if CSIDL <> 0 then begin
        if SHGetSpecialFolderLocation(GetDesktopWindow, CSIDL,
          PIDL) = NOERROR then begin
            SetLength(Result, MAX_PATH);
            if SHGetPathFromIDList(PIDL, PChar(Result)) then
              SetLength(Result, StrLen(PChar(Result)))
            else
              Result := '';
        end
        else
          Result := '';
      end
      else
        Result := '';
    end;
  end;
end;

////////////////////////////////////////////////////////////////////////////////

Procedure MarkText(CustomMemo: TCustomMemo; Start, Len: Integer);
begin
{start := start+1 jetz?}
  SendMessage(CustomMemo.Handle, EM_SETSEL, Start, Start+Len);
  SendMessage(CustomMemo.Handle, WM_KEYDOWN, VK_RIGHT, 0);
  SendMessage(CustomMemo.Handle, EM_SETSEL, Start, Start+Len);
end;

////////////////////////////////////////////////////////////////////////////////

function GetFirst(CustomMemo: TCustomMemo; Range: TSearchRange): Integer;
begin
  if Range = srWholeTxt then
    Result := 0
  else
    if (Range = srFromCursor) or (Range = srMarkedTxt) then
      Result := CustomMemo.SelStart
    else
      Result := -1;
end;

////////////////////////////////////////////////////////////////////////////////

function GetLast(CustomMemo: TCustomMemo; Range: TSearchRange): Integer;
begin
  if (Range = srWholeTxt) or (Range = srFromCursor) then
    Result := Length(CustomMemo.Text)
  else
    if Range = srMarkedTxt then
      Result := Length(CustomMemo.Text) -
         (Length(CustomMemo.Text) - CustomMemo.SelLength) + CustomMemo.SelStart
    else
      Result := -1;
end;

////////////////////////////////////////////////////////////////////////////////

function StringSearch(const Source, SearchStr: String;
   const First, Last: Integer; Flags: TRichSearchTypes): Integer;
{ in JCL you'll find StrSearch which is written in ASM }
{ Also look for FindInText in JVCL }
{ stWholeWord, stMatchCase are used
  stBackward, stSetSelection are ignored }
var
  Found: Integer;
  stext, spattern: String;
  stmp: String;
const AlphaNum = ['a'..'z', 'A'..'Z', '0'..'9', '_'];
begin
  { default: not found }
  Result := -1;

  { take care of stMatchCase in Flags }
  if (stMatchCase in Flags) then begin
    stext := Copy(Source, First + 1, Last - First);
    spattern := SearchStr;
  end
  else begin
    stext := AnsiUpperCase(Copy(Source, First + 1, Last - First));
    spattern := AnsiUpperCase(SearchStr);
  end;

  Found := Pos(spattern, stext);
  if Found > 0 then begin
    { Result is based on zero }
    Result := First + Found - 1;

    { take care of stWholeWord in Flags }
    if (stWholeWord in Flags) then begin
      if Result = First then
        stmp := Copy(Source, Result, Length(SearchStr) + 1)
      else
        stmp := Copy(Source, Result, Length(SearchStr) + 2);
      if ( ( (stmp[1] in AlphaNum) and not(Result = First) ) or
           ( (stmp[Length(stmp)] in AlphaNum) and
             not(Result+Length(spattern) = Last) ) ) then begin
        Found := StringSearch(stext, spattern, Found + Length(stmp) - 4,
           Last, Flags);
        if Found > 0 then
          Result := First + Found
        else
          Result := -1;
      end
      else
        { is WholeWord, so Result is already ok };
    end
    else
      { WholeWord doesn't matter, so Result is already ok };
  end;
end;

////////////////////////////////////////////////////////////////////////////////

function StringReplace(const S, OldPattern, NewPattern: String;
   Flags: TReplaceFlags {$IFDEF SUPPORTS_DEFAULTPARAMS}= [rfReplaceAll]{$ENDIF})
   : String; {$IFDEF DELPHI4_UP}overload;{$ENDIF}
{
written by Bjrge Sther
Newsgroup: comp.lang.pascal.delphi.misc
Date: Thu, 8 Nov 2001 16:09:07 +0100

"As a result of a long discussion in this NG a few months ago
I created this replace function. It behaves differently depending on whether
the replacement String is longer, shorter or of equal length.
The function is compatible with StringReplace, only much faster."
}
var i              : Integer;
    SearchStr, Patt: String;
    ChStart, ChPatt: PChar;
    OldPattLen     : Integer;
    NewPattLen     : Integer;
    Delta          : Integer;
begin
  result := S;
  OldPattLen := Length(OldPattern);
  NewPattLen := Length(NewPattern);
  if rfIgnoreCase in Flags then begin
    SearchStr := ANSIUppercase(S);
    Patt := ANSIUpperCase(OldPattern);
  end
  else begin
    SearchStr := S;
    Patt := OldPattern;
  end;
  if OldPattLen > NewPattLen then begin { We may do only one String truncation }
    ChStart := PChar(SearchStr);
    ChPatt := ChStart;
    Delta := 0;
    while true do begin
      ChPatt := StrPos(ChPatt, PChar(Patt));
      if ChPatt = nil then
        break;
      if NewPattLen > 0 then
        Move(NewPattern[1], result[ChPatt-ChStart+1-Delta], NewPattLen);
      Move(result[(ChPatt-ChStart-Delta)+OldPattLen+1],
         result[ChPatt-ChStart+NewPattLen+1-Delta],
         Length(Result)-(ChPatt-ChStart-Delta)-OldPattLen);
      Delta := Delta+OldPattLen-NewPattLen; { Keep track of difference in
                                              positions between S and result }
      if not (rfReplaceAll in Flags) then
        break;
      inc(ChPatt);
    end;
    SetLength(Result, Length(Result)-Delta);
  end
  else
    if OldPattLen < NewPattLen then begin { The slowest, needs to enlarge String
                                            for each match }
      NewPattLen := Length(NewPattern);
      ChStart := PChar(SearchStr);
      ChPatt := ChStart;
      Delta := 0;
      while true do begin
        ChPatt := StrPos(ChPatt, PChar(Patt));
        if ChPatt = nil then
          break;
        SetLength(Result, Length(Result)+NewPattLen-OldPattLen);
        Move(result[(ChPatt-ChStart)+Delta+OldPattLen+1],
           result[(ChPatt-ChStart)+(NewPattLen)+Delta+1],
           Length(Result)-(ChPatt-ChStart)-Delta-NewPattLen);
        Move(NewPattern[1], result[ChPatt-ChStart+1+Delta], NewPattLen);
        Delta := Delta+(NewPattLen-OldPattLen); { Keep track of difference in
                                                positions between S and result }
        if not (rfReplaceAll in Flags) then
          break;
        inc(ChPatt);
      end;
    end
    else
      if (OldPattLen = 1) and (NewPattLen = 1) then begin { Just replace Chars }
        for i := 1 to Length(SearchStr) do
          if SearchStr[i] = Patt[1] then begin
            result[i] := NewPattern[1];
            if not (rfReplaceAll in Flags) then
              break;
          end;
      end
      else begin { Equal length, but more than 1 char,
                   so keep length, but move chars }
        ChStart := PChar(SearchStr);
        ChPatt := ChStart;
        while true do begin
          ChPatt := StrPos(ChPatt, PChar(Patt));
          if ChPatt = nil then
            break;
          Move(NewPattern[1], result[ChPatt-ChStart+1], OldPattLen);
          if not (rfReplaceAll in Flags) then
            break;
          inc(ChPatt);
        end;
      end;
end;

////////////////////////////////////////////////////////////////////////////////

function GetExeForProtocol(URL: String): String;
var Reg : TRegistry;
    Res : Boolean;
    Temp: String;
    P   : Integer;
begin
  Result := '';
  P := Pos(':', URL);
  if P>1 then
    delete(URL, P, length(URL)-P+1);
  {$IFDEF Delphi4_Up}
  Reg := TRegistry.Create(Key_Read);
  {$else}
  Reg := TRegistry.Create;
  {$ENDIF}
  try
    Reg.Rootkey := HKEY_CLASSES_ROOT;
    Res := Reg.OpenKey(URL+'\shell\open\command', false);
    if Res then begin
      Temp := Reg.ReadString('');
      while (length(Temp) > 0) and ((Temp[1] = '"') or (Temp[1] = ' ')) do
        delete(Temp, 1, 1);
      P := Pos('"', Temp);
      if P > 0 then
        delete(Temp, P, length(Temp)-P+1);
      Result := Temp;
      Reg.CloseKey;
    end;
  finally
    Reg.Free;
  end;
end;

////////////////////////////////////////////////////////////////////////////////

procedure GetVersionInfo(const aFileName: String; aVersionList: TStrings);
type
  PTransBuffer = ^TTransBuffer;
  TTransBuffer = Array[1..4] of SmallInt;
var
  i:        TFileInfoStr;
  infoSize: DWORD;
  ptrans:   PTransBuffer;
  transStr, typeStr: String;
  value:    PChar;
  verBuf:   pointer;
  verSize, wnd:  DWORD;
begin
  AVersionList.Clear;
  infoSize := GetFileVersioninfoSize(PChar(aFileName), wnd);
  if infoSize <> 0 then begin
    GetMem(verBuf, infoSize);
    try
      if GetFileVersionInfo(PChar(aFileName), wnd, infoSize, verBuf) then begin
        VerQueryvalue(verBuf, PChar('\VarFileInfo\Translation'),
           Pointer(ptrans), verSize);
        transStr := IntToHex(ptrans^[1], 4) + IntToHex(ptrans^[2], 4);
        for i := Low(TFileInfoStr) to High(TFileInfoStr) do begin
          typeStr := 'StringFileInfo\' + transStr + '\' + FileInfoStr[i];
          if VerQueryvalue(verBuf, PChar(typeStr), Pointer(value), verSize) then
            aVersionList.Add(value)
          else
            aVersionList.Add('');
        end
      end
      else
        raise MFException.Create(srEReadingVersionInfo);
    finally
      FreeMem(verBuf);
    end;
  end
  else
    raise MFException.CreateFmt(srENoVersionInfo, [aFileName]);
end;

end.
