{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

Initial Autor   : Marcus Fuchs

Last Modified at: 12. January 2003
Last Modified by: Marcus Fuchs

You may retrieve the latest version of this file at the MFLib home page,
located at http://www.mfhome.net/mflib/
-------------------------------------------------------------------------------}

unit MFSearch;

{******************************************************************************}
{                                                                              }
{  This Form is intended to be used from an other unit (MFDialogs)             }
{                                                                              }
{******************************************************************************}

// remove the period to enable Debug settings (see MFLib.inc)
{.$DEFINE DEBUG}

{$I MFLib.inc}

interface

uses
  Windows, Classes, Controls, Forms, Dialogs, StdCtrls,                { VCL   }
  MFTypes;                                                             { MFLib }

type
  TMFSearchDlg = class(TForm)
    FindStrEdit: TEdit;
    MatchCaseCBox: TCheckBox;
    WholeWordCBox: TCheckBox;
    TxtRBtn: TRadioButton;
    CursorRBtn: TRadioButton;
    MarkedRBtn: TRadioButton;
    SearchBtn: TButton;
    CancelBtn: TButton;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure MatchCaseCBoxClick(Sender: TObject);
    procedure WholeWordCBoxClick(Sender: TObject);
    procedure TxtRBtnClick(Sender: TObject);
    procedure CursorRBtnClick(Sender: TObject);
    procedure MarkedRBtnClick(Sender: TObject);
  private
    FStyle: TFindDlgStyle; { No differences between dsLast and dsCustom }
    FRange: TSearchRange;
    FOptions: TRichSearchTypes;
    Procedure SetRange(Value: TSearchRange);
    Procedure SetOptions(Values: TRichSearchTypes);
  public
    property Style: TFindDlgStyle read FStyle write FStyle default dsEmpty;
    property Range: TSearchRange read FRange write SetRange default srWholeTxt;
    property Options: TRichSearchTypes read FOptions write SetOptions default [];
  end;

var
  MFSearchDlg: TMFSearchDlg;


implementation

{$R *.DFM}

uses
  MFStrRes;

procedure TMFSearchDlg.FormCreate(Sender: TObject);
begin
  Self.Caption := srMFSearchDlg;
  Label1.Caption := srSearchforL;
  MatchCaseCBox.Caption := srMatchCaseCBox;
  WholeWordCBox.Caption := srWholeWordCBox;
  TxtRBtn.Caption := srTxtRBtn;
  CursorRBtn.Caption := srCursorRBtn;
  MarkedRBtn.Caption := srMarkedRBtn;
  SearchBtn.Caption := srSearchBtn;
  CancelBtn.Caption := srCancelBtn;
  GroupBox1.Caption := srOptionsGB;

  FStyle := dsEmpty;
  FRange := srWholeTxt;
  FOptions := [];
end;

procedure TMFSearchDlg.FormShow(Sender: TObject);
begin
  Case FStyle of
    dsEmpty : Begin
                FindStrEdit.Text := '';
                MatchCaseCBox.Checked := False;
                WholeWordCBox.Checked := False;
                Range := srWholeTxt;
                TxtRBtn.Checked := True;
                FindStrEdit.SetFocus;
              End;
    dsCustom: SearchBtn.SetFocus;
    dsLast  : SearchBtn.SetFocus;
  End;
end;

Procedure TMFSearchDlg.SetRange(Value: TSearchRange);
Begin
  If FRange <> Value Then Begin
    FRange := Value;
    Case Value of
      srWholeTxt  : TxtRBtn.Checked := True;
      srFromCursor: CursorRBtn.Checked := True;
      srMarkedTxt : MarkedRBtn.Checked := True;
    End;
  End;
End;

procedure TMFSearchDlg.SetOptions(Values: TRichSearchTypes);
Begin
  If FOptions <> Values Then Begin
    FOptions := Values;
    MatchCaseCBox.Checked := (stMatchCase in Values);
    WholeWordCBox.Checked := (stWholeWord in Values);
  End;
End;

procedure TMFSearchDlg.MatchCaseCBoxClick(Sender: TObject);
begin
  If MatchCaseCBox.Checked Then
    FOptions := FOptions+[stMatchCase]
  Else
    FOptions := FOptions-[stMatchCase];
end;

procedure TMFSearchDlg.WholeWordCBoxClick(Sender: TObject);
begin
  If WholeWordCBox.Checked Then
    FOptions := FOptions+[stWholeWord]
  Else
    FOptions := FOptions-[stWholeWord];
end;

procedure TMFSearchDlg.TxtRBtnClick(Sender: TObject);
begin
  FRange := srWholeTxt;
end;

procedure TMFSearchDlg.CursorRBtnClick(Sender: TObject);
begin
  FRange := srFromCursor;
end;

procedure TMFSearchDlg.MarkedRBtnClick(Sender: TObject);
begin
  FRange := srMarkedTxt;
end;

end.
