{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

Initial Autor   : Marcus Fuchs

Last Modified at: 05. February 2003
Last Modified by: Marcus Fuchs

You may retrieve the latest version of this file at the MFLib home page,
located at http://www.mfhome.net/mflib/
-------------------------------------------------------------------------------}

unit MFTypes;

interface { ------------------------------------------------------ interface - }

// remove the period to enable Debug settings (see MFLib.inc)
{.$DEFINE DEBUG}

{$I MFLib.inc}

uses
  SysUtils, Classes, Graphics;                                         { VCL   }

type
  MFException = class(Exception);
{ Summary: Basic exeptionclass in MFib. }
{ Description: MFException is the basicclass for all Exceptions implemented in
  MFLib. }
  EMFVersionConflict = class(MFException);
{ Summary: EMFVersionConflict is the Exception-class for Versionconflicts. }
{ Description: EMFVersionConflict is raised if an application tries to handle
  files, or what ever, that had been created, modified or used by another
  version of this application. It could also be raised if a file, or what ever
  doesn't have the versioninfo expected. }

type
  TSearchRange = (srWholeTxt, srFromCursor, srMarkedTxt);
  { Description: }
  { Parameters:
    srWholeText   Search the whole text.
    srFromCursor  Start the search where the cursor is.
    srMarkedText  Search within the text currently marked. }
  TFindDlgStyle = (dsEmpty, dsCustom, dsLast);
  { Description: TFindDlgStyle is designed for Dialogs which need userinput,
    like SearchDlg or ReplaceDlg. It won't be helpfull for e.g. an AboutBox. }
  TMFAboutDlgStyle = (dsClassic, dsModern);

{$IFNDEF DELPHI4_UP}
type
  TReplaceFlags = set of (rfReplaceAll, rfIgnoreCase);
  { Description: }
{$ENDIF}

type
  TRichSearchType = (stWholeWord, stMatchCase, stBackward, stSetSelection);
  { Note: TRichSearchType is the some as in RxRichEd.pas (RxLib) or
    JvRichEd.pas (JVCL 2.0), so you can typecast if needed. }
  TRichSearchTypes = set of TRichSearchType;
  { Note: TRichSearchTypes is the some as in RxRichEd.pas (RxLib) or
    JvRichEd.pas (JVCL 2.0), so you can typecast if needed. }

type
  TMFGradientStyle = (gsHorizontal, gsVertical, gsElliptic, gsRectangle,
                      gsVertCenter, gsHorizCenter, gsLeftTop, gsRightTop,
                      gsLeftTopRightBottom, gsRightTopLeftBottom, gsNone);

type
  TMFTextStyle = (tsNone, tsRaised, tsHeavyRaised, tsLowered, tsHeavyLowered,
    tsShadow);
  { Overview:  }
  { Description:  }

type
  TMFFrameLayout = (flNormal, flFlat);
  { Overview: Indicates how to draw a 3D border. }
  { Description: If set to flNormal the border looks like the one of a standard
    Button else it would look like SpeedButton with Flat=True. }

type
  TMFGradient = class(TPersistent)
  { Overview:  }
  { Description:  }
  private
    FBeginColor: TColor;
    FEndColor: TColor;
    FStyle: TMFGradientStyle;
  protected
    procedure SetBeginColor(Value: TColor);
    procedure SetEndColor(Value: TColor);
    procedure SetStyle(Value: TMFGradientStyle);
  public
    OnChanged: TNotifyEvent;
    constructor Create; virtual;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property BeginColor: TColor read FBeginColor write SetBeginColor
       default clYellow;
    property EndColor: TColor read FEndColor write SetEndColor
       default clRed;
    property Style: TMFGradientStyle read FStyle write SetStyle
       default gsHorizCenter;
  end;

type
  TMFMouseColors = class(TPersistent)
  { Overview:  }
  { Description: With TMFMouseColors it is possible to define colors depending
    on mouseposition. If it is out of the control Color is used. If the mouse is
    over the control, while it is not pressed HoverColor should be used. And if
    one of the mousebuttons is pressed DownColor should be used. }
  private
    FColor: TColor;
    FHoverColor: TColor;
    FDownColor: TColor;
  protected
    procedure SetColor(Value: TColor);
    procedure SetHoverColor(Value: TColor);
    procedure SetDownColor(Value: TColor);
  public
    OnChanged: TNotifyEvent;
    constructor Create; virtual;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property Color: TColor read FColor write SetColor default clWindowText;
    property HoverColor: TColor read FHoverColor write SetHoverColor default clHighlightText;
    property DownColor: TColor read FDownColor write SetDownColor default clGrayText;
  end;

type
  TMFTextColors = class(TPersistent)
  { Overview:  }
  { Description:  }
  private
    FColors: TMFMouseColors;
    FDisabledColor: TColor;
    FLightColors: TMFMouseColors;
    FShadowColors: TMFMouseColors;
  protected
    procedure SetColors(Value: TMFMouseColors);
    procedure SetDisabledColor(Value: TColor);
    procedure SetLightColors(Value: TMFMouseColors);
    procedure SetShadowColors(Value: TMFMouseColors);
  public
    OnChanged: TNotifyEvent;
    constructor Create; virtual;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property Colors: TMFMouseColors read FColors write SetColors;
    property DisabledColor: TColor read FDisabledColor write SetDisabledColor
       default clGrayText;
    property LightColors: TMFMouseColors read FLightColors write SetLightColors;
    property ShadowColors: TMFMouseColors read FShadowColors write SetShadowColors;
  end;

type
  TMFBorderColors = class(TPersistent)
  { Overview:  }
  { Description:  }
  private
    FLightColors: TMFMouseColors;
    FShadowColors: TMFMouseColors;
    FDkShadowColors: TMFMouseColors;
  protected
    procedure SetLightColors(Value: TMFMouseColors);
    procedure SetShadowColors(Value: TMFMouseColors);
    procedure SetDkShadowColors(Value: TMFMouseColors);
  public
    OnChanged: TNotifyEvent;
    constructor Create; virtual;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property LightColors: TMFMouseColors read FLightColors write SetLightColors;
    property ShadowColors: TMFMouseColors read FShadowColors
       write SetShadowColors;
    property DkShadowColors: TMFMouseColors read FDkShadowColors
       write SetDkShadowColors;
  end;


implementation { -------------------------------------------- implementation - }

{ - TMFGradient -------------------------------------------------------------- }

constructor TMFGradient.Create;
begin
  FBeginColor := clYellow;
  FEndColor := clRed;
  FStyle := gsHorizCenter;
  inherited;
end;

destructor TMFGradient.Destroy;
begin
  inherited;
end;

procedure TMFGradient.Assign(Source: TPersistent);
begin
  if Source is TMFGradient then begin
    with TMFGradient(Source) do begin
      if self.FBeginColor <> FBeginColor then
        self.FBeginColor := FBeginColor;
      if self.FEndColor <> FEndColor then
        self.FEndColor := FEndColor;
      if self.FStyle <> FStyle then
        self.FStyle := FStyle;
    end;
    if Assigned(OnChanged) then
      OnChanged(self);
  end
  else
    inherited Assign(Source);
end;

procedure TMFGradient.SetBeginColor(Value: TColor);
begin
  if FBeginColor <> Value then begin
    FBeginColor := Value;
    if Assigned(OnChanged) then
      OnChanged(self);
  end;
end;

procedure TMFGradient.SetEndColor(Value: TColor);
begin
  if FEndColor <> Value then begin
    FEndColor := Value;
    if Assigned(OnChanged) then
      OnChanged(self);
  end;
end;

procedure TMFGradient.SetStyle(Value: TMFGradientStyle);
begin
  if FStyle <> Value then begin
    FStyle := Value;
    if Assigned(OnChanged) then
      OnChanged(self);
  end;
end;

{ - TMFMouseColors ----------------------------------------------------------- }

constructor TMFMouseColors.Create;
begin
  inherited;
  FColor := clWindowText;
  FHoverColor := clHighlightText;
  FDownColor := clGrayText;
end;

destructor TMFMouseColors.Destroy;
begin
  inherited;
end;

procedure TMFMouseColors.Assign(Source: TPersistent);
begin
  if Source is TMFMouseColors then begin
    with TMFMouseColors(Source) do begin
      if self.FColor <> FColor then
        self.FColor := FColor;
      if self.FHoverColor <> FHoverColor then
        self.FHoverColor := FHoverColor;
      if self.FHoverColor <> FDownColor then
        self.FDownColor := FDownColor;
    end;
    if Assigned(OnChanged) then
      OnChanged(self);
  end
  else
    inherited Assign(Source);
end;

procedure TMFMouseColors.SetColor(Value: TColor);
begin
  if FColor <> Value then begin
    FColor := Value;
    if Assigned(OnChanged) then
      OnChanged(self);
  end;
end;

procedure TMFMouseColors.SetHoverColor(Value: TColor);
begin
  if FHoverColor <> Value then begin
    FHoverColor := Value;
    if Assigned(OnChanged) then
      OnChanged(self);
  end;
end;

procedure TMFMouseColors.SetDownColor(Value: TColor);
begin
  if FDownColor <> Value then begin
    FDownColor := Value;
    if Assigned(OnChanged) then
      OnChanged(self);
  end;
end;

{ - TMFTextColors ----------------------------------------------------------- }

constructor TMFTextColors.Create;
begin
  inherited;
  FColors := TMFMouseColors.Create;
  FLightColors := TMFMouseColors.Create;
  FShadowColors := TMFMouseColors.Create;

  FColors.Color := clWindowText;
  FColors.HoverColor := clWindowText;
  FColors.DownColor := clWindowText;

  FDisabledColor := clGrayText;

  FLightColors.Color := clHighlightText;
  FLightColors.HoverColor := clHighlightText;
  FLightColors.DownColor := clHighlightText;

  FShadowColors.Color := clGrayText;
  FShadowColors.HoverColor := clGrayText;
  FShadowColors.DownColor := clGrayText;
end;

destructor TMFTextColors.Destroy;
begin
  FShadowColors.Free;
  FLightColors.Free;
  FColors.Free;
  inherited;
end;

procedure TMFTextColors.Assign(Source: TPersistent);
begin
  if Source is TMFTextColors then begin
    with TMFTextColors(Source) do begin
      if self.FColors <> FColors then
        self.FColors.Assign(FColors);
      if self.FDisabledColor <> FDisabledColor then
        self.FDisabledColor := FDisabledColor;
      if self.FLightColors <> FLightColors then
        self.FLightColors.Assign(FLightColors);
      if self.FShadowColors <> FShadowColors then
        self.FShadowColors.Assign(FShadowColors);
    end;
    if Assigned(OnChanged) then
      OnChanged(self);
  end
  else
    inherited Assign(Source);
end;

procedure TMFTextColors.SetColors(Value: TMFMouseColors);
begin
  if FColors <> Value then begin
    FColors.Assign(Value);
    if Assigned(OnChanged) then
      OnChanged(self);
  end;
end;

procedure TMFTextColors.SetDisabledColor(Value: TColor);
begin
  if FDisabledColor <> Value then begin
    FDisabledColor := Value;
    if Assigned(OnChanged) then
      OnChanged(self);
  end;
end;

procedure TMFTextColors.SetLightColors(Value: TMFMouseColors);
begin
  if FLightColors <> Value then begin
    FLightColors.Assign(Value);
    if Assigned(OnChanged) then
      OnChanged(self);
  end;
end;

procedure TMFTextColors.SetShadowColors(Value: TMFMouseColors);
begin
  if FShadowColors <> Value then begin
    FShadowColors.Assign(Value);
    if Assigned(OnChanged) then
      OnChanged(self);
  end;
end;

{ - TMFBorderColors --------------------------------------------------------- }

constructor TMFBorderColors.Create;
begin
  inherited;
  FLightColors := TMFMouseColors.Create;
  FShadowColors := TMFMouseColors.Create;
  FDkShadowColors := TMFMouseColors.Create;

  FLightColors.Color := clBtnHighlight;
  FLightColors.HoverColor := clBtnHighlight;
  FLightColors.DownColor := clBtnHighlight;
  FShadowColors.Color := clBtnShadow;
  FShadowColors.HoverColor := clBtnShadow;
  FShadowColors.DownColor := clBtnShadow;
  FDkShadowColors.Color := cl3DDkShadow;
  FDkShadowColors.HoverColor := cl3DDkShadow;
  FDkShadowColors.DownColor := cl3DDkShadow;
end;

destructor TMFBorderColors.Destroy;
begin
  FDkShadowColors.Free;
  FShadowColors.Free;
  FLightColors.Free;
end;

procedure TMFBorderColors.Assign(Source: TPersistent);
begin
  if Source is TMFBorderColors then begin
    with TMFBorderColors(Source) do begin
      if self.FLightColors <> FLightColors then
        self.FLightColors.Assign(FLightColors);
      if self.FShadowColors <> FShadowColors then
        self.FShadowColors.Assign(FShadowColors);
      if self.FDkShadowColors <> FShadowColors then
        self.FDkShadowColors.Assign(FDkShadowColors);
    end;
    if Assigned(OnChanged) then
      OnChanged(self);
  end
  else
    inherited Assign(Source);
end;

procedure TMFBorderColors.SetLightColors(Value: TMFMouseColors);
begin
  if FLightColors <> Value then begin
    FLightColors.Assign(Value);
    if Assigned(OnChanged) then
      OnChanged(self);
  end;
end;

procedure TMFBorderColors.SetShadowColors(Value: TMFMouseColors);
begin
  if FShadowColors <> Value then begin
    FShadowColors.Assign(Value);
    if Assigned(OnChanged) then
      OnChanged(self);
  end;
end;

procedure TMFBorderColors.SetDkShadowColors(Value: TMFMouseColors);
begin
  if FDkShadowColors <> Value then begin
    FDkShadowColors.Assign(Value);
    if Assigned(OnChanged) then
      OnChanged(self);
  end;
end;

end.
